@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -82,17 +82,12 @@ pub type Result<T> = std::result::Result<T, Error>;
 pub use builder::DirProviderBuilder;
 
 #[cfg(all(feature = "onion-service-client", feature = "experimental-api"))]
-#[cfg_attr(
-    docsrs,
-    doc(cfg(all(feature = "onion-service-client", feature = "experimental-api")))
-)]
 pub use {
     tor_hscrypto::pk::{HsClientDescEncKey, HsId},
     tor_keymgr::KeystoreSelector,
 };
 
 #[cfg(feature = "geoip")]
-#[cfg_attr(docsrs, doc(cfg(feature = "geoip")))]
 pub use tor_geoip::CountryCode;
 
 /// Return a list of the protocols [supported](tor_protover::doc_supported) by this crate.
@@ -159,7 +154,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
