@@ -1,25 +1,37 @@
 //! Types and functions to configure a Tor Relay.
-//!
-//! NOTE: At the moment, only StorageConfig is implemented but as we ramp up arti relay
-//! implementation, more configurations will show up.
+
+// TODO: It would be nice to remove the builder aspect of these config objects, as we don't need
+// them for arti-relay. But I don't think we can do so while still using tor-config. See:
+// https://gitlab.torproject.org/tpo/core/arti/-/issues/2253
+
+mod listen;
 
 use std::borrow::Cow;
+use std::net::{SocketAddrV4, SocketAddrV6};
 use std::path::PathBuf;
 
 use derive_builder::Builder;
 use derive_more::AsRef;
-
 use directories::ProjectDirs;
 use fs_mistrust::{Mistrust, MistrustBuilder};
 use serde::{Deserialize, Serialize};
 use std::sync::LazyLock;
 use tor_chanmgr::{ChannelConfig, ChannelConfigBuilder};
-use tor_config::{ConfigBuildError, impl_standard_builder, mistrust::BuilderExt};
+use tor_circmgr::{CircuitTiming, PathConfig, PreemptiveCircuitConfig};
+use tor_config::{ConfigBuildError, ExplicitOrAuto, impl_standard_builder, mistrust::BuilderExt};
 use tor_config_path::{CfgPath, CfgPathError, CfgPathResolver};
+use tor_dircommon::config::{NetworkConfig, NetworkConfigBuilder};
+use tor_dircommon::fallback::FallbackList;
+use tor_guardmgr::bridge::BridgeConfig;
+use tor_guardmgr::{VanguardConfig, VanguardConfigBuilder, VanguardMode};
 use tor_keymgr::config::{ArtiKeystoreConfig, ArtiKeystoreConfigBuilder};
 use tracing::metadata::Level;
 use tracing_subscriber::filter::EnvFilter;
 
+use crate::util::NonEmptyList;
+
+use self::listen::Listen;
+
 /// Paths used for default configuration files.
 pub(crate) fn default_config_paths() -> Result<Vec<PathBuf>, CfgPathError> {
     // the base path resolver includes the 'ARTI_RELAY_CONFIG' variable
@@ -46,8 +58,6 @@ pub(crate) fn default_config_paths() -> Result<Vec<PathBuf>, CfgPathError> {
 ///     An arti-specific cache directory.
 ///   - `ARTI_RELAY_CONFIG`:
 ///     An arti-specific configuration directory.
-///   - `ARTI_RELAY_SHARED_DATA`:
-///     An arti-specific directory in the user's "shared data" space.
 ///   - `ARTI_RELAY_LOCAL_DATA`:
 ///     An arti-specific directory in the user's "local data" space.
 ///   - `PROGRAM_DIR`:
@@ -60,10 +70,15 @@ pub(crate) fn default_config_paths() -> Result<Vec<PathBuf>, CfgPathError> {
 /// and so should use appropriate system-specific overrides under the hood.
 /// (Some of those overrides are based on environment variables.)
 /// For more information, see that crate's documentation.
+//
+// NOTE: We intentionally don't expose an `ARTI_RELAY_SHARED_DATA`
+// (analogous to `ARTI_SHARED_DATA` in arti).
+// This is almost certainly never intended over `ARTI_RELAY_LOCAL_DATA`,
+// so by removing it we don't need to worry about bugs from mixing them up.
+// We can introduce it later if really needed.
 pub(crate) fn base_resolver() -> CfgPathResolver {
     let arti_relay_cache = project_dirs().map(|x| Cow::Owned(x.cache_dir().to_owned()));
     let arti_relay_config = project_dirs().map(|x| Cow::Owned(x.config_dir().to_owned()));
-    let arti_relay_shared_data = project_dirs().map(|x| Cow::Owned(x.data_dir().to_owned()));
     let arti_relay_local_data = project_dirs().map(|x| Cow::Owned(x.data_local_dir().to_owned()));
     let program_dir = get_program_dir().map(Cow::Owned);
     let user_home = tor_config_path::home().map(Cow::Borrowed);
@@ -72,7 +87,6 @@ pub(crate) fn base_resolver() -> CfgPathResolver {
 
     resolver.set_var("ARTI_RELAY_CACHE", arti_relay_cache);
     resolver.set_var("ARTI_RELAY_CONFIG", arti_relay_config);
-    resolver.set_var("ARTI_RELAY_SHARED_DATA", arti_relay_shared_data);
     resolver.set_var("ARTI_RELAY_LOCAL_DATA", arti_relay_local_data);
     resolver.set_var("PROGRAM_DIR", program_dir);
     resolver.set_var("USER_HOME", user_home);
@@ -98,16 +112,25 @@ fn project_dirs() -> Result<&'static ProjectDirs, CfgPathError> {
 
 /// A configuration used by a TorRelay.
 ///
-/// Most users will create a TorRelayConfig by running
-/// [`TorRelayConfig::default`].
-///
-/// Finally, you can get fine-grained control over the members of a
-/// TorRelayConfig using [`TorRelayConfigBuilder`].
+/// This is a builder so that it works with tor-config.
+/// We don't expect to ever use it as a builder since we don't provide this as a public rust API.
 #[derive(Clone, Builder, Debug, Eq, PartialEq, AsRef)]
 #[builder(build_fn(error = "ConfigBuildError"))]
 #[builder(derive(Serialize, Deserialize, Debug))]
 #[non_exhaustive]
 pub(crate) struct TorRelayConfig {
+    /// Configuration for the "relay" part of the relay.
+    // TODO: Add a better doc comment here once we figure out exactly how we want the config to be
+    // structured.
+    #[builder(sub_builder)]
+    #[builder_field_attr(serde(default))]
+    pub(crate) relay: RelayConfig,
+
+    /// Information about the Tor network we want to connect to.
+    #[builder(sub_builder)]
+    #[builder_field_attr(serde(default))]
+    pub(crate) tor_network: NetworkConfig,
+
     /// Logging configuration
     #[builder(sub_builder)]
     #[builder_field_attr(serde(default))]
@@ -127,13 +150,116 @@ pub(crate) struct TorRelayConfig {
     #[builder(sub_builder)]
     #[builder_field_attr(serde(default))]
     pub(crate) system: SystemConfig,
+
+    /// Information about how to build paths through the network.
+    // We don't expose this field in the config.
+    #[builder(setter(skip))]
+    #[builder_field_attr(serde(skip))]
+    #[builder(default)]
+    // Needed to implement `CircMgrConfig`.
+    #[as_ref]
+    pub(crate) path_rules: PathConfig,
+
+    /// Information about vanguards.
+    // We don't expose this field in the config.
+    #[builder(setter(skip))]
+    #[builder_field_attr(serde(skip))]
+    #[builder(default = r#"
+        VanguardConfigBuilder::default()
+            .mode(ExplicitOrAuto::Explicit(VanguardMode::Disabled))
+            .build()
+            .expect("Could not build a disabled `VanguardConfig`")"#)]
+    // Needed to implement `CircMgrConfig`.
+    #[as_ref]
+    pub(crate) vanguards: VanguardConfig,
+
+    /// Information about how to retry and expire circuits and request for circuits.
+    // We don't expose this field in the config.
+    #[builder(setter(skip))]
+    #[builder_field_attr(serde(skip))]
+    #[builder(default)]
+    // Needed to implement `CircMgrConfig`.
+    #[as_ref]
+    pub(crate) circuit_timing: CircuitTiming,
+
+    /// Information about preemptive circuits.
+    // We don't expose this field in the config.
+    #[builder(setter(skip))]
+    #[builder_field_attr(serde(skip))]
+    #[builder(default)]
+    // Needed to implement `CircMgrConfig`.
+    #[as_ref]
+    pub(crate) preemptive_circuits: PreemptiveCircuitConfig,
 }
-impl_standard_builder! { TorRelayConfig }
+impl_standard_builder! { TorRelayConfig: !Default }
 
 impl tor_config::load::TopLevel for TorRelayConfig {
     type Builder = TorRelayConfigBuilder;
 }
 
+impl tor_circmgr::CircMgrConfig for TorRelayConfig {}
+
+// Needed to implement `GuardMgrConfig`.
+impl AsRef<FallbackList> for TorRelayConfig {
+    fn as_ref(&self) -> &FallbackList {
+        self.tor_network.fallback_caches()
+    }
+}
+
+// Needed to implement `GuardMgrConfig`.
+impl AsRef<[BridgeConfig]> for TorRelayConfig {
+    fn as_ref(&self) -> &[BridgeConfig] {
+        // Relays don't use bridges.
+        &[]
+    }
+}
+
+impl tor_guardmgr::GuardMgrConfig for TorRelayConfig {
+    fn bridges_enabled(&self) -> bool {
+        // Relays don't use bridges.
+        false
+    }
+}
+
+/// Configuration for the "relay" part of the relay.
+///
+/// TODO: I'm not really sure what to call this yet. I'm expecting that we'll rename and reorganize
+/// things as we add more options. But we should come back to this and update the name and/or doc
+/// comment.
+///
+/// TODO: There's a high-level issue for discussing these options:
+/// <https://gitlab.torproject.org/tpo/core/arti/-/issues/2252>
+#[derive(Debug, Clone, Builder, Eq, PartialEq)]
+#[builder(build_fn(error = "ConfigBuildError"))]
+#[builder(derive(Debug, Serialize, Deserialize))]
+pub(crate) struct RelayConfig {
+    /// Addresses to listen on for incoming OR connections.
+    pub(crate) listen: Listen,
+
+    /// Addresses to advertise on the network for receiving OR connections.
+    // For now, we've decided that we don't want to include any IP address auto-detection in
+    // arti-relay, so we require users to provide the addresses to advertise. (So no `Option` and
+    // `builder(default)` here).
+    pub(crate) advertise: Advertise,
+}
+impl_standard_builder! { RelayConfig: !Default }
+
+/// The address(es) to advertise on the network.
+// TODO: We'll want to make sure we check that the addresses are valid before uploading them in a
+// server descriptor (for example no `INADDR_ANY`, multicast, etc). We can't do that validation here
+// during parsing, since we don't know exactly which addresses are valid or not. For example we
+// don't know if local addresses are allowed as we don't know here whether the user plans to run a
+// testing tor network. We also don't want to do the validation too late (for example when uploading
+// the server descriptor) as it's better to validate at startup. A better place might be to perform
+// the validation in the `RelayConfig` builder validate.
+#[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
+pub(crate) struct Advertise {
+    /// All relays must advertise an IPv4 address.
+    ipv4: NonEmptyList<SocketAddrV4>,
+    /// Relays may optionally advertise an IPv6 address.
+    ipv6: Vec<SocketAddrV6>,
+}
+
 /// Default log level.
 pub(crate) const DEFAULT_LOG_LEVEL: Level = Level::INFO;
 
@@ -194,6 +320,9 @@ pub(crate) struct StorageConfig {
     /// in it may be deleted outside of the control of Arti,
     /// and Arti will continue to function properly.
     /// It is also fine to delete the directory as a whole, while Arti is not running.
+    ///
+    /// Should be accessed through the `cache_dir()` getter to provide better error messages when
+    /// resolving the path.
     //
     // Usage note, for implementations of Arti components:
     //
@@ -238,13 +367,21 @@ impl StorageConfig {
         &self.permissions
     }
 
-    /// Return the fully expanded path of the cache directory.
+    /// Return the fully expanded path of the state directory.
     pub(crate) fn state_dir(
         &self,
         resolver: &CfgPathResolver,
     ) -> Result<PathBuf, ConfigBuildError> {
         resolve_cfg_path(&self.state_dir, "state_dir", resolver)
     }
+
+    /// Return the fully expanded path of the cache directory.
+    pub(crate) fn cache_dir(
+        &self,
+        resolver: &CfgPathResolver,
+    ) -> Result<PathBuf, ConfigBuildError> {
+        resolve_cfg_path(&self.cache_dir, "cache_dir", resolver)
+    }
 }
 
 /// Configuration for system resources used by the relay.
@@ -293,39 +430,18 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
     use super::*;
 
-    #[test]
-    fn defaults() {
-        let dflt = TorRelayConfig::default();
-        let b2 = TorRelayConfigBuilder::default();
-        let dflt2 = b2.build().unwrap();
-        assert_eq!(&dflt, &dflt2);
-    }
-
-    #[test]
-    fn builder() {
-        let mut bld = TorRelayConfigBuilder::default();
-        bld.storage()
-            .cache_dir(CfgPath::new("/var/tmp/foo".to_owned()))
-            .state_dir(CfgPath::new("/var/tmp/bar".to_owned()));
-
-        let val = bld.build().unwrap();
-
-        assert_ne!(val, TorRelayConfig::default());
-    }
-
     fn cfg_variables() -> impl IntoIterator<Item = (&'static str, PathBuf)> {
         let project_dirs = project_dirs().unwrap();
         let list = [
             ("ARTI_RELAY_CACHE", project_dirs.cache_dir()),
             ("ARTI_RELAY_CONFIG", project_dirs.config_dir()),
-            ("ARTI_RELAY_SHARED_DATA", project_dirs.data_dir()),
             ("ARTI_RELAY_LOCAL_DATA", project_dirs.data_local_dir()),
             ("PROGRAM_DIR", &get_program_dir().unwrap()),
             ("USER_HOME", tor_config_path::home().unwrap()),
