@@ -6,6 +6,7 @@ use paste::paste;
 
 use derive_builder::Builder;
 use serde::{Deserialize, Serialize};
+use tor_config_path::CfgPath;
 
 #[cfg(feature = "onion-service-service")]
 use crate::onion_proxy::{
@@ -14,7 +15,6 @@ use crate::onion_proxy::{
 #[cfg(not(feature = "onion-service-service"))]
 use crate::onion_proxy_disabled::{OnionServiceProxyConfigMap, OnionServiceProxyConfigMapBuilder};
 #[cfg(feature = "rpc")]
-#[cfg_attr(docsrs, doc(cfg(feature = "rpc")))]
 pub use crate::rpc::{RpcConfig, RpcConfigBuilder};
 use arti_client::TorClientConfig;
 #[cfg(feature = "onion-service-service")]
@@ -164,6 +164,24 @@ pub struct ProxyConfig {
 }
 impl_standard_builder! { ProxyConfig }
 
+/// Configuration for arti-specific storage locations.
+///
+/// See also [`arti_client::config::StorageConfig`].
+#[derive(Debug, Clone, Builder, Eq, PartialEq)]
+#[builder(build_fn(error = "ConfigBuildError"))]
+#[builder(derive(Debug, Serialize, Deserialize))]
+pub struct ArtiStorageConfig {
+    /// A file in which to write information about the ports we're listening on.
+    #[builder(setter(into), default = "default_port_info_file()")]
+    pub(crate) port_info_file: CfgPath,
+}
+impl_standard_builder! { ArtiStorageConfig }
+
+/// Return the default ports_info_file location.
+fn default_port_info_file() -> CfgPath {
+    CfgPath::new("${ARTI_LOCAL_DATA}/public/port_info.json".to_owned())
+}
+
 /// Configuration for system resources used by Tor.
 ///
 /// You cannot change *these variables* in this section on a running Arti client.
@@ -262,6 +280,14 @@ pub struct ArtiConfig {
     #[builder_field_attr(serde(default))]
     pub(crate) system: SystemConfig,
 
+    /// Information on where things are stored by Arti.
+    ///
+    /// Note that [`TorClientConfig`] also has a storage configuration;
+    /// our configuration logic should merge them correctly.
+    #[builder(sub_builder(fn_name = "build"))]
+    #[builder_field_attr(serde(default))]
+    pub(crate) storage: ArtiStorageConfig,
+
     /// Configured list of proxied onion services.
     ///
     /// Note that this field is present unconditionally, but when onion service
@@ -364,6 +390,11 @@ impl ArtiConfig {
         &self.proxy
     }
 
+    /// Return the [`ArtiStorageConfig`] for this configuration.
+    pub fn storage(&self) -> &ArtiStorageConfig {
+        &self.storage
+    }
+
     /// Return the [`RpcConfig`] for this configuration.
     #[cfg(feature = "rpc")]
     pub fn rpc(&self) -> &RpcConfig {
@@ -382,7 +413,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -569,6 +600,8 @@ mod test {
                 "proxy.socks_listen",
                 "proxy.dns_listen",
                 "use_obsolete_software",
+                "circuit_timing.disused_circuit_timeout",
+                "storage.port_info_file",
             ],
         );
 
@@ -628,6 +661,17 @@ mod test {
             ],
         );
 
+        declare_exceptions(
+            None,
+            None, // it's there, but not formatted for auto-testing
+            FeatureDependent,
+            &[
+                // Settings only available with tokio-console support
+                "logging.tokio_console",
+                "logging.tokio_console.enabled",
+            ],
+        );
+
         declare_exceptions(
             None,
             None, // it's there, but not formatted for auto-testing
