@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -67,7 +67,7 @@ macro_rules! semipublic_mod {
     {
         $(
             $( #[$meta:meta] )*
-            mod $name:ident ;
+            $dflt_vis:vis mod $name:ident ;
         )*
     }  => {
         $(
@@ -76,7 +76,7 @@ macro_rules! semipublic_mod {
                 if #[cfg(feature="experimental-api")] {
                    pub mod $name;
                 } else {
-                   mod $name;
+                   $dflt_vis mod $name;
                 }
             }
          )*
