@@ -7,13 +7,9 @@ use std::str::FromStr;
 
 use anyhow::Result;
 
-use arti_client::{InertTorClient, TorClient, TorClientBuilder, TorClientConfig};
+use arti_client::{InertTorClient, TorClient, TorClientConfig};
 use clap::{ArgMatches, Args, FromArgMatches, Parser, Subcommand};
-use safelog::DisplayRedacted;
-use tor_keymgr::{
-    CTorPath, KeyMgr, KeyPath, KeystoreEntry, KeystoreEntryResult, KeystoreId,
-    UnrecognizedEntryError,
-};
+use tor_keymgr::{KeyMgr, KeystoreEntry, KeystoreEntryResult, KeystoreId, UnrecognizedEntryError};
 use tor_rtcompat::Runtime;
 
 use crate::{ArtiConfig, subcommands::prompt};
@@ -33,6 +29,7 @@ pub(crate) enum KeysSubcommands {
     Keys(KeysSubcommand),
 }
 
+/// The `keys` subcommand.
 #[derive(Subcommand, Debug, Clone)]
 pub(crate) enum KeysSubcommand {
     /// List keys and certificates.
@@ -129,22 +126,33 @@ pub(crate) fn run<R: Runtime>(
     match subcommand {
         KeysSubcommand::List(args) => run_list_keys(&args, &client_builder.create_inert()?),
         KeysSubcommand::ListKeystores => run_list_keystores(&client_builder.create_inert()?),
-        KeysSubcommand::CheckIntegrity(args) => {
-            run_check_integrity(&args, &client_builder, &rt, config, client_config)
-        }
+        KeysSubcommand::CheckIntegrity(args) => run_check_integrity(
+            &args,
+            &rt.reenter_block_on(client_builder.create_bootstrapped())?,
+            config,
+            client_config,
+        ),
     }
 }
 
 /// Print information about a keystore entry.
 fn display_entry(entry: &KeystoreEntry, keymgr: &KeyMgr) {
-    match entry.key_path() {
-        KeyPath::Arti(_) => display_arti_entry(entry, keymgr),
-        KeyPath::CTor(path) => display_ctor_entry(entry, path),
-        unrecognized => {
-            eprintln!(
-                "WARNING: unexpected `tor_keymgr::KeyPath` variant encountered: {:?}",
-                unrecognized
-            );
+    let raw_entry = entry.raw_entry();
+    match keymgr.describe(entry.key_path()) {
+        Some(e) => {
+            println!(" Keystore ID: {}", entry.keystore_id());
+            println!(" Role: {}", e.role());
+            println!(" Summary: {}", e.summary());
+            println!(" KeystoreItemType: {:?}", entry.key_type());
+            println!(" Location: {}", raw_entry.raw_id());
+            let extra_info = e.extra_info();
+            println!(" Extra info:");
+            for (key, value) in extra_info {
+                println!(" - {key}: {value}");
+            }
+        }
+        None => {
+            println!(" Unrecognized path {}", raw_entry.raw_id());
         }
     }
     println!("\n {}", "-".repeat(LINE_LEN));
@@ -223,16 +231,11 @@ fn run_list_keystores(client: &InertTorClient) -> Result<()> {
 /// Run `keys check-integrity` subcommand.
 fn run_check_integrity<R: Runtime>(
     args: &CheckIntegrityArgs,
-    builder: &TorClientBuilder<R>,
-    runtime: &R,
+    client: &TorClient<R>,
     config: &ArtiConfig,
     client_config: &TorClientConfig,
 ) -> Result<()> {
-    let inert_client = builder.create_inert()?;
-    let client = runtime.reenter_block_on(builder.create_bootstrapped())?;
-
-    // TODO: `TorClient` should have a `KeyMgr` accessor.
-    let keymgr = inert_client.keymgr()?;
+    let keymgr = client.keymgr()?;
 
     let keystore_ids = match &args.keystore_id {
         Some(id) => vec![id.to_owned()],
@@ -254,7 +257,7 @@ fn run_check_integrity<R: Runtime>(
             // `service` cannot be dropped as long as `expired_entries` is in use, since
             // `expired_entries` holds references to `services`.
             let services = create_all_services(config, client_config)?;
-            let mut expired_entries: Vec<_> = get_expired_keys(&services, &client)?;
+            let mut expired_entries: Vec<_> = get_expired_keys(&services, client)?;
         }
     }
 
@@ -385,56 +388,6 @@ fn display_keystore_entries(
     }
 }
 
-/// Displays an Arti native keystore entry.
-fn display_arti_entry(entry: &KeystoreEntry, keymgr: &KeyMgr) {
-    let raw_entry = entry.raw_entry();
-    match keymgr.describe(entry.key_path()) {
-        Ok(e) => {
-            println!(" Keystore ID: {}", entry.keystore_id());
-            println!(" Role: {}", e.role());
-            println!(" Summary: {}", e.summary());
-            println!(" KeystoreItemType: {:?}", entry.key_type());
-            println!(" Location: {}", raw_entry.raw_id());
-            let extra_info = e.extra_info();
-            println!(" Extra info:");
-            for (key, value) in extra_info {
-                println!(" - {key}: {value}");
-            }
-        }
-        Err(_) => {
-            println!(" Unrecognized path {}", raw_entry.raw_id());
-        }
-    }
-}
-
-/// Displays a CTor keystore entry.
-///
-/// This function outputs the details of a CTor keystore entry, distinguishing
-/// between client and service keys based on [`CTorPath`].
-fn display_ctor_entry(entry: &KeystoreEntry, path: &CTorPath) {
-    let raw_entry = entry.raw_entry();
-    match path {
-        CTorPath::ClientHsDescEncKey(id) => {
-            println!(" CTor client key");
-            println!(" Hidden service ID: {}", id.display_unredacted());
-        }
-        CTorPath::Service { nickname, path: _ } => {
-            println!(" CTor service key");
-            println!(" Hidden service nickname: {}", nickname);
-        }
-        unrecognized => {
-            eprintln!(
-                "WARNING: unexpected `tor_keymgr::CTorPath` variant encountered: {:?}",
-                unrecognized
-            );
-            return;
-        }
-    }
-    println!(" Keystore ID: {}", entry.keystore_id());
-    println!(" KeystoreItemType: {:?}", entry.key_type());
-    println!(" Location: {}", raw_entry.raw_id());
-}
-
 /// Helper function for `run_check_integrity`.
 ///
 /// Creates an [`OnionService`] for each configured hidden service.
