@@ -72,9 +72,7 @@ pub(crate) async fn connect_via_proxy<R: NetStreamProvider + Send + Sync>(
 ) -> Result<R::Stream, ProxyError> {
     trace!(
         "Launching a proxied connection to {} via proxy at {} using {:?}",
-        target,
-        proxy,
-        protocol
+        target, proxy, protocol
     );
     let mut stream = runtime
         .connect(proxy)
@@ -123,7 +121,7 @@ pub(crate) async fn connect_via_proxy<R: NetStreamProvider + Send + Sync>(
             NS::Finished(fin) => {
                 break fin
                     .into_output_forbid_pipelining()
-                    .map_err(ProxyError::SocksProto)?
+                    .map_err(ProxyError::SocksProto)?;
             }
             NS::Recv(mut recv) => {
                 let n = stream.read(recv.buf()).await?;
@@ -135,8 +133,7 @@ pub(crate) async fn connect_via_proxy<R: NetStreamProvider + Send + Sync>(
     let status = reply.status();
     trace!(
         "SOCKS handshake with {} succeeded, with status {:?}",
-        proxy,
-        status
+        proxy, status
     );
 
     if status != SocksStatus::SUCCEEDED {
@@ -204,8 +201,8 @@ impl From<std::io::Error> for ProxyError {
 
 impl tor_error::HasKind for ProxyError {
     fn kind(&self) -> tor_error::ErrorKind {
-        use tor_error::ErrorKind as EK;
         use ProxyError as E;
+        use tor_error::ErrorKind as EK;
         match self {
             E::ProxyConnect(_) | E::ProxyIo(_) => EK::LocalNetworkError,
             E::InvalidSocksAddr(_) | E::InvalidSocksRequest(_) => EK::BadApiUsage,
@@ -220,9 +217,9 @@ impl tor_error::HasKind for ProxyError {
 
 impl tor_error::HasRetryTime for ProxyError {
     fn retry_time(&self) -> tor_error::RetryTime {
-        use tor_error::RetryTime as RT;
         use ProxyError as E;
         use SocksStatus as S;
+        use tor_error::RetryTime as RT;
         match self {
             E::ProxyConnect(_) | E::ProxyIo(_) => RT::AfterWaiting,
             E::InvalidSocksAddr(_) => RT::Never,
@@ -244,7 +241,6 @@ impl tor_error::HasRetryTime for ProxyError {
 }
 
 #[cfg(feature = "pt-client")]
-#[cfg_attr(docsrs, doc(cfg(feature = "pt-client")))]
 /// An object that connects to a Tor bridge via an external pluggable transport
 /// that provides a proxy.
 #[derive(Clone, Debug)]
@@ -258,7 +254,6 @@ pub struct ExternalProxyPlugin<R> {
 }
 
 #[cfg(feature = "pt-client")]
-#[cfg_attr(docsrs, doc(cfg(feature = "pt-client")))]
 impl<R: NetStreamProvider + Send + Sync> ExternalProxyPlugin<R> {
     /// Make a new `ExternalProxyPlugin`.
     pub fn new(rt: R, proxy_addr: SocketAddr, proxy_version: SocksVersion) -> Self {
@@ -283,14 +278,14 @@ impl<R: NetStreamProvider + Send + Sync> TransportImplHelper for ExternalProxyPl
             ChannelMethod::Direct(_) => {
                 return Err(crate::Error::UnusableTarget(bad_api_usage!(
                     "Used pluggable transport for a TCP connection."
-                )))
+                )));
             }
             ChannelMethod::Pluggable(target) => target,
             other => {
                 return Err(crate::Error::UnusableTarget(bad_api_usage!(
                     "Used unknown, unsupported, transport {:?} for a TCP connection.",
                     other,
-                )))
+                )));
             }
         };
 
@@ -408,7 +403,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
