@@ -39,6 +39,7 @@ impl<B: BackoffSchedule, R: Runtime> Runner<B, R> {
     /// If `fallible_fn` eventually returns `Ok(_)`, return that output. Otherwise,
     /// keep retrying until either `fallible_fn` has failed too many times, or until
     /// a fatal error occurs.
+    #[allow(clippy::cognitive_complexity)] // TODO: Refactor
     pub(super) async fn run<T, E, F>(
         mut self,
         mut fallible_fn: impl FnMut() -> F,
@@ -100,7 +101,7 @@ impl<B: BackoffSchedule, R: Runtime> Runner<B, R> {
                                 "failed to {}: {e}", self.doing
                             );
 
-                            errors.push(e.clone());
+                            errors.push_timed(e.clone(), self.runtime.now(), None);
                             (e.should_retry(), self.schedule.next_delay(&e))
                         }
                         Err(e) => {
@@ -235,7 +236,7 @@ mod tests {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -434,8 +435,7 @@ mod tests {
         run_test(
             None,
             BackoffWithMaxRetries,
-            iter::repeat(Transient)
-                .take(RETRIES_UNTIL_FATAL)
+            std::iter::repeat_n(Transient, RETRIES_UNTIL_FATAL)
                 .chain([Fatal])
                 .chain(iter::repeat(Transient)),
             EXPECTED_TOTAL_RUNS,
