@@ -6,8 +6,10 @@
 use crate::internal_prelude::*;
 
 use tor_cell::relaycell::msg::{Connected, End, Introduce2};
+use tor_circmgr::ServiceOnionServiceDataTunnel;
 use tor_hscrypto::Subcredential;
-use tor_proto::stream::{IncomingStream, IncomingStreamRequest};
+use tor_keymgr::ArtiPath;
+use tor_proto::client::stream::{IncomingStream, IncomingStreamRequest};
 
 /// Request to complete an introduction/rendezvous handshake.
 ///
@@ -39,6 +41,8 @@ pub struct RendRequest {
     ///
     /// TODO: This also contains `raw`, which is maybe not so great; it would be
     /// neat to implement more efficiently.
+    //
+    // TODO MSRV TBD: Replace with std OnceCell (#1996)
     expanded: once_cell::unsync::OnceCell<rend_handshake::IntroRequest>,
 }
 
@@ -56,7 +60,7 @@ pub struct StreamRequest {
     stream: IncomingStream,
 
     /// The circuit that made this request.
-    on_circuit: Arc<ClientCirc>,
+    on_tunnel: Arc<ServiceOnionServiceDataTunnel>,
 }
 
 /// Keys and objects needed to answer a RendRequest.
@@ -107,7 +111,10 @@ impl RendRequestContext {
             .list_matching(&pattern)?
             .iter()
             .map(|entry| -> Result<Option<_>, FatalError> {
-                let path = entry.key_path();
+                let path = entry
+                    .key_path()
+                    .arti()
+                    .ok_or_else(|| internal!("CTorPath matched arti pattern?!"))?;
                 let matches = path
                     .matches(&pattern)
                     .ok_or_else(|| internal!("path matched but no longer does?!"))?;
@@ -132,16 +139,10 @@ impl RendRequestContext {
 
     /// Try to parse the `captures` of `path` as a [`TimePeriod`].
     fn parse_time_period(
-        path: &KeyPath,
-        captures: &[KeyPathRange],
+        path: &ArtiPath,
+        captures: &[ArtiPathRange],
     ) -> Result<TimePeriod, tor_keymgr::Error> {
-        use tor_keymgr::{KeyPathError, KeystoreCorruptionError as KCE};
-
-        let path = match path {
-            KeyPath::Arti(path) => path,
-            KeyPath::CTor(_) => todo!(),
-            _ => todo!(),
-        };
+        use tor_keymgr::{ArtiPathError, KeyPathError, KeystoreCorruptionError as KCE};
 
         let [denotator] = captures else {
             return Err(internal!(
@@ -155,18 +156,22 @@ impl RendRequestContext {
             return Err(internal!("captured substring out of range?!").into());
         };
 
-        let slug = Slug::new(denotator.to_string()).map_err(|e| {
-            KCE::KeyPath(KeyPathError::InvalidArtiPath {
-                path: path.clone(),
-                error: e.into(),
+        let tp = (|| {
+            let slug = Slug::new(denotator.to_string())
+                .map_err(|e| ArtiPathError::InvalidArtiPath(e.into()))?;
+
+            TimePeriod::from_slug(&slug).map_err(|error| {
+                ArtiPathError::InvalidKeyPathComponentValue {
+                    key: "time_period".to_owned(),
+                    value: slug.clone(),
+                    error,
+                }
             })
-        })?;
-        let tp = TimePeriod::from_slug(&slug).map_err(|error| {
-            KCE::KeyPath(KeyPathError::InvalidKeyPathComponentValue {
-                key: "time_period".to_owned(),
+        })()
+        .map_err(|err| {
+            KCE::KeyPath(KeyPathError::Arti {
                 path: path.clone(),
-                value: slug,
-                error,
+                err,
             })
         })?;
 
@@ -191,7 +196,7 @@ impl RendRequest {
 
     /// Try to return a reference to the intro_request, creating it if it did
     /// not previously exist.
-    fn intro_request(
+    pub(crate) fn intro_request(
         &self,
     ) -> Result<&rend_handshake::IntroRequest, rend_handshake::IntroRequestError> {
         self.expanded.get_or_try_init(|| {
@@ -213,7 +218,7 @@ impl RendRequest {
             .expect("intro_request succeeded but did not fill 'expanded'.");
         let rend_handshake::OpenSession {
             stream_requests,
-            circuit,
+            tunnel,
         } = intro_request
             .establish_session(
                 self.context.filter.clone(),
@@ -223,6 +228,8 @@ impl RendRequest {
             .await
             .map_err(ClientError::EstablishSession)?;
 
+        let tunnel = Arc::new(tunnel);
+
         // Note that we move circuit (which is an Arc<ClientCirc>) into this
         // closure, which lives for as long as the stream of StreamRequest, and
         // for as long as each individual StreamRequest.  This is how we keep
@@ -230,7 +237,7 @@ impl RendRequest {
         // the Stream we return is dropped.
         Ok(stream_requests.map(move |stream| StreamRequest {
             stream,
-            on_circuit: circuit.clone(),
+            on_tunnel: tunnel.clone(),
         }))
     }
 
@@ -278,7 +285,7 @@ impl StreamRequest {
     /// Reject this request and close the rendezvous circuit entirely,
     /// along with all other streams attached to the circuit.
     pub fn shutdown_circuit(self) -> Result<(), Bug> {
-        self.on_circuit.terminate();
+        self.on_tunnel.terminate();
         Ok(())
     }
 
