@@ -11,7 +11,7 @@ use std::ops::Deref;
 use std::sync::Arc;
 
 use crate::raw::RawKeystoreEntry;
-use crate::{KeyPathError, KeystoreId};
+use crate::{KeyPath, KeyPathError, KeystoreId};
 
 /// An Error type for this crate.
 #[derive(thiserror::Error, Debug, Clone)]
@@ -103,10 +103,14 @@ pub enum ArtiPathSyntaxError {
 #[error("Keystore corruption")]
 #[non_exhaustive]
 pub enum KeystoreCorruptionError {
-    /// A keystore contains a key that has an invalid [`KeyPath`](crate::KeyPath).
+    /// A keystore contains a key that has an invalid [`KeyPath`].
     #[error("{0}")]
     KeyPath(#[from] KeyPathError),
 
+    /// A keystore contains an unrecognized [`KeyPath`].
+    #[error("Unrecognized key path {0}")]
+    Unrecognized(KeyPath),
+
     /// Missing certificate for key.
     #[error("Missing certificate for key")]
     MissingCertificate,
@@ -181,7 +185,7 @@ mod tests {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
