@@ -12,17 +12,17 @@ pub(crate) mod plain;
 pub(crate) mod vote;
 
 use super::{ConsensusFlavor, ConsensusMethods};
-use crate::doc;
 use crate::doc::netstatus::NetstatusKwd;
-use crate::doc::netstatus::{IgnoredPublicationTimeSp, Protocols, RelayFlags, RelayWeight};
+use crate::doc::netstatus::{IgnoredPublicationTimeSp, Protocols, RelayWeight};
 use crate::parse::parser::Section;
 use crate::types::misc::*;
+use crate::types::relay_flags::{self, DocRelayFlags, RelayFlag, RelayFlags};
 use crate::types::version::TorVersion;
-use crate::util::intern::InternCache;
 use crate::{Error, NetdocErrorKind as EK, Result};
 use itertools::chain;
 use std::sync::Arc;
 use std::{net, time};
+use tor_basic_utils::intern::InternCache;
 use tor_error::internal;
 use tor_llcrypto::pk::rsa::RsaIdentity;
 
