@@ -40,57 +40,45 @@ use crate::types::misc::*;
 use crate::types::policy::*;
 use crate::types::version::TorVersion;
 use crate::util::PeekableIterator;
-use crate::{doc, AllowAnnotations, Error, NetdocErrorKind as EK, Result};
+use crate::{AllowAnnotations, Error, KeywordEncodable, NetdocErrorKind as EK, Result};
 
 use ll::pk::ed25519::Ed25519Identity;
-use once_cell::sync::Lazy;
 use std::sync::Arc;
+use std::sync::LazyLock;
 use std::{net, time};
 use tor_cert::CertType;
-use tor_checkable::{signed, timed, Timebound};
+use tor_checkable::{Timebound, signed, timed};
 use tor_error::{internal, into_internal};
 use tor_llcrypto as ll;
 use tor_llcrypto::pk::rsa::RsaIdentity;
 
 use digest::Digest;
 
+/// Length of a router descriptor digest
+pub const DOC_DIGEST_LEN: usize = 20;
+
 /// The digest of a RouterDesc document, as reported in a NS consensus.
-pub type RdDigest = [u8; 20];
+pub type RdDigest = [u8; DOC_DIGEST_LEN];
 
 /// A router descriptor, with possible annotations.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
+#[non_exhaustive]
 pub struct AnnotatedRouterDesc {
     /// Annotation for this router descriptor; possibly empty.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ann: RouterAnnotation,
+    pub ann: RouterAnnotation,
     /// Underlying router descriptor; signatures not checked yet.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    router: UncheckedRouterDesc,
+    pub router: UncheckedRouterDesc,
 }
 
 /// Annotations about a router descriptor, as stored on disc.
-#[allow(dead_code)] // don't warn about fields not getting read.
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
 #[derive(Default)]
+#[non_exhaustive]
 pub struct RouterAnnotation {
     /// Description of where we got this router descriptor
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    source: Option<String>,
+    pub source: Option<String>,
     /// When this descriptor was first downloaded.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    downloaded: Option<time::SystemTime>,
+    pub downloaded: Option<time::SystemTime>,
     /// Description of what we're willing to use this descriptor for.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    purpose: Option<String>,
+    pub purpose: Option<String>,
 }
 
 /// Information about a relay, parsed from a router descriptor.
@@ -106,89 +94,65 @@ pub struct RouterAnnotation {
 ///
 /// Before using this type to connect to a relay, you MUST check that
 /// it is valid, using is_expired_at().
-#[allow(dead_code)] // don't warn about fields not getting read.
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
 #[derive(Clone, Debug)]
+#[non_exhaustive]
 pub struct RouterDesc {
     /// Human-readable nickname for this relay.
     ///
     /// This is not secure, and not guaranteed to be unique.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    nickname: Nickname,
+    pub nickname: Nickname,
     /// IPv4 address for this relay.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ipv4addr: Option<net::Ipv4Addr>,
+    pub ipv4addr: Option<net::Ipv4Addr>,
     /// IPv4 ORPort for this relay.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    orport: u16,
+    pub orport: u16,
     /// IPv6 address and port for this relay.
     // TODO: we don't use a socketaddrv6 because we don't care about
     // the flow and scope fields.  We should decide whether that's a
     // good idea.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ipv6addr: Option<(net::Ipv6Addr, u16)>,
+    pub ipv6addr: Option<(net::Ipv6Addr, u16)>,
     /// Directory port for contacting this relay for direct HTTP
     /// directory downloads.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    dirport: u16,
+    pub dirport: u16,
     /// Declared uptime for this relay, in seconds.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    uptime: Option<u64>,
+    pub uptime: Option<u64>,
     /// Time when this router descriptor was published.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    published: time::SystemTime,
+    pub published: time::SystemTime,
     /// Ed25519 identity certificate (identity key authenticating a
     /// signing key)
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    identity_cert: tor_cert::Ed25519Cert,
+    pub identity_cert: tor_cert::Ed25519Cert,
     /// RSA identity key for this relay. (Deprecated; never use this without
     /// the ed25519 identity as well).
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    rsa_identity_key: ll::pk::rsa::PublicKey,
+    pub rsa_identity_key: ll::pk::rsa::PublicKey,
     /// RSA identity key for this relay. (Deprecated; never use this without
     /// the ed25519 identity as well).
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    rsa_identity: ll::pk::rsa::RsaIdentity,
+    pub rsa_identity: ll::pk::rsa::RsaIdentity,
     /// Key for extending a circuit to this relay using the ntor protocol.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ntor_onion_key: ll::pk::curve25519::PublicKey,
+    pub ntor_onion_key: ll::pk::curve25519::PublicKey,
     /// Key for extending a circuit to this relay using the
     /// (deprecated) TAP protocol.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    tap_onion_key: Option<ll::pk::rsa::PublicKey>,
+    pub tap_onion_key: Option<ll::pk::rsa::PublicKey>,
     /// List of subprotocol versions supported by this relay.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    proto: Arc<tor_protover::Protocols>,
+    pub proto: tor_protover::Protocols,
     /// True if this relay says it's a directory cache.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    is_dircache: bool,
+    pub is_dircache: bool,
     /// True if this relay says that it caches extrainfo documents.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    is_extrainfo_cache: bool,
+    pub is_extrainfo_cache: bool,
     /// Declared family members for this relay.  If two relays are in the
     /// same family, they shouldn't be used in the same circuit.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    family: Arc<RelayFamily>,
+    pub family: Arc<RelayFamily>,
     /// Declared (and proven) family IDs for this relay. If two relays
     /// share a family ID, they shouldn't be used in the same circuit.
     family_ids: Vec<RelayFamilyId>,
     /// Software and version that this relay says it's running.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    platform: Option<RelayPlatform>,
+    pub platform: Option<RelayPlatform>,
     /// A complete address-level policy for which IPv4 addresses this relay
     /// says it supports.
     // TODO: these polices can get bulky too. Perhaps we should
     // de-duplicate them too.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ipv4_policy: AddrPolicy,
+    pub ipv4_policy: AddrPolicy,
     /// A summary of which ports this relay is willing to connect to
     /// on IPv6.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ipv6_policy: Arc<PortPolicy>,
+    pub ipv6_policy: Arc<PortPolicy>,
 }
 
 /// Description of the software a relay is running.
@@ -258,7 +222,7 @@ decl_keyword! {
 }
 
 /// Rules for parsing a set of router descriptor annotations.
-static ROUTER_ANNOTATIONS: Lazy<SectionRules<RouterKwd>> = Lazy::new(|| {
+static ROUTER_ANNOTATIONS: LazyLock<SectionRules<RouterKwd>> = LazyLock::new(|| {
     use RouterKwd::*;
 
     let mut rules = SectionRules::builder();
@@ -273,7 +237,7 @@ static ROUTER_ANNOTATIONS: Lazy<SectionRules<RouterKwd>> = Lazy::new(|| {
 });
 /// Rules for tokens that are allowed in the first part of a
 /// router descriptor.
-static ROUTER_HEADER_RULES: Lazy<SectionRules<RouterKwd>> = Lazy::new(|| {
+static ROUTER_HEADER_RULES: LazyLock<SectionRules<RouterKwd>> = LazyLock::new(|| {
     use RouterKwd::*;
 
     let mut rules = SectionRules::builder();
@@ -285,7 +249,7 @@ static ROUTER_HEADER_RULES: Lazy<SectionRules<RouterKwd>> = Lazy::new(|| {
 });
 /// Rules for  tokens that are allowed in the first part of a
 /// router descriptor.
-static ROUTER_BODY_RULES: Lazy<SectionRules<RouterKwd>> = Lazy::new(|| {
+static ROUTER_BODY_RULES: LazyLock<SectionRules<RouterKwd>> = LazyLock::new(|| {
     use RouterKwd::*;
 
     let mut rules = SectionRules::builder();
@@ -329,7 +293,7 @@ static ROUTER_BODY_RULES: Lazy<SectionRules<RouterKwd>> = Lazy::new(|| {
 });
 
 /// Rules for items that appear at the end of a router descriptor.
-static ROUTER_SIG_RULES: Lazy<SectionRules<RouterKwd>> = Lazy::new(|| {
+static ROUTER_SIG_RULES: LazyLock<SectionRules<RouterKwd>> = LazyLock::new(|| {
     use RouterKwd::*;
 
     let mut rules = SectionRules::builder();
@@ -393,7 +357,7 @@ impl RouterDesc {
     /// Return a reference to the list of subprotocol versions supported by this
     /// relay.
     pub fn protocols(&self) -> &tor_protover::Protocols {
-        self.proto.as_ref()
+        &self.proto
     }
 
     /// Return a reference to this relay's Ntor onion key.
@@ -459,7 +423,7 @@ impl RouterDesc {
     /// Does not actually check liveness or signatures; you need to do that
     /// yourself before you can do the output.
     pub fn parse(s: &str) -> Result<UncheckedRouterDesc> {
-        let mut reader = crate::parse::tokenize::NetDocReader::new(s);
+        let mut reader = crate::parse::tokenize::NetDocReader::new(s)?;
         let result = Self::parse_internal(&mut reader).map_err(|e| e.within(s))?;
         // We permit empty lines at the end of router descriptors, since there's
         // a known issue in Tor relays that causes them to return them this way.
@@ -539,7 +503,7 @@ impl RouterDesc {
         // Legacy RSA identity
         let rsa_identity_key: ll::pk::rsa::PublicKey = body
             .required(SIGNING_KEY)?
-            .parse_obj::<RsaPublic>("RSA PUBLIC KEY")?
+            .parse_obj::<RsaPublicParse1Helper>("RSA PUBLIC KEY")?
             .check_len_eq(1024)?
             .check_exponent(65537)?
             .into();
@@ -638,7 +602,7 @@ impl RouterDesc {
         // TAP key
         let tap_onion_key: Option<ll::pk::rsa::PublicKey> = if let Some(tok) = body.get(ONION_KEY) {
             Some(
-                tok.parse_obj::<RsaPublic>("RSA PUBLIC KEY")?
+                tok.parse_obj::<RsaPublicParse1Helper>("RSA PUBLIC KEY")?
                     .check_len_eq(1024)?
                     .check_exponent(65537)?
                     .into(),
@@ -669,12 +633,10 @@ impl RouterDesc {
         // List of subprotocol versions
         let proto = {
             let proto_tok = body.required(PROTO)?;
-            doc::PROTOVERS_CACHE.intern(
-                proto_tok
-                    .args_as_str()
-                    .parse::<tor_protover::Protocols>()
-                    .map_err(|e| EK::BadArgument.at_pos(proto_tok.pos()).with_source(e))?,
-            )
+            proto_tok
+                .args_as_str()
+                .parse::<tor_protover::Protocols>()
+                .map_err(|e| EK::BadArgument.at_pos(proto_tok.pos()).with_source(e))?
         };
 
         // tunneled-dir-server
@@ -762,7 +724,7 @@ impl RouterDesc {
                         return Err(Error::from(internal!(
                             "tried to parse a strange line as a policy"
                         ))
-                        .at_pos(ruletok.pos()))
+                        .at_pos(ruletok.pos()));
                     }
                 };
                 let pat: AddrPortPattern = ruletok
@@ -898,10 +860,10 @@ fn advance_to_next_routerdesc(reader: &mut NetDocReader<'_, RouterKwd>, annotate
 
 impl<'a> RouterReader<'a> {
     /// Construct a RouterReader to take router descriptors from a string.
-    pub fn new(s: &'a str, allow: &AllowAnnotations) -> Self {
-        let reader = NetDocReader::new(s);
+    pub fn new(s: &'a str, allow: &AllowAnnotations) -> Result<Self> {
+        let reader = NetDocReader::new(s)?;
         let annotated = allow == &AllowAnnotations::AnnotationsAllowed;
-        RouterReader { annotated, reader }
+        Ok(RouterReader { annotated, reader })
     }
 
     /// Extract an annotation from this reader.
@@ -969,7 +931,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -1059,8 +1021,8 @@ mod test {
 
     #[test]
     fn test_bad() {
-        use crate::types::policy::PolicyError;
         use crate::Pos;
+        use crate::types::policy::PolicyError;
         fn check(fname: &str, e: &Error) {
             let text = read_bad(fname);
             let rd = RouterDesc::parse(&text);
@@ -1142,7 +1104,7 @@ mod test {
 ";
         s += &read_bad("mismatched-fp");
 
-        let rd = RouterReader::new(&s, &AllowAnnotations::AnnotationsAllowed);
+        let rd = RouterReader::new(&s, &AllowAnnotations::AnnotationsAllowed).unwrap();
         let v: Vec<_> = rd.collect();
         assert!(v[0].is_err());
         assert!(v[1].is_ok());
