@@ -2,43 +2,11 @@
 
 use super::*;
 
-/// Macro to help check that netdoc items in a derive input are in the right order
-///
-/// Used only by the `NetdocParseable` derive-deftly macro.
-#[doc(hidden)]
-#[macro_export]
-macro_rules! netdoc_ordering_check {
-    { } => { compile_error!("netdoc must have an intro item so cannot be empty"); };
-
-    // When we have   K0 P0 K1 P1 ...
-    //   * Check that P0 and P1 have a consistent ordr
-    //   * Continue with   K1 P1 ...
-    // So we check each consecutive pair of fields.
-    { $k0:ident $f0:ident $k1:ident $f1:ident $($rest:tt)* } => {
-        $crate::netdoc_ordering_check! { <=? $k0 $k1 $f1 }
-        $crate::netdoc_ordering_check! { $k1 $f1 $($rest)* }
-    };
-    { $k0:ident $f0:ident } => {}; // finished
-
-    // Individual ordering checks for K0 <=? K1
-    //
-    // We write out each of the allowed this-kind next-kind combinations:
-    { <=? intro     $any:ident $f1:ident } => {};
-    { <=? normal    normal     $f1:ident } => {};
-    { <=? normal    subdoc     $f1:ident } => {};
-    { <=? subdoc    subdoc     $f1:ident } => {};
-    // Not in the allowed list, must be an error:
-    { <=? $k0:ident $k1:ident  $f1:ident } => {
-        compile_error!(concat!(
-            "in netdoc, ", stringify!($k1)," field ", stringify!($f1),
-            " may not come after ", stringify!($k0),
-        ));
-    };
-}
-
 /// Helper to implemnet `dtrace!` inside `NetdocParseable` derive-deftly macro.
 #[doc(hidden)]
 pub fn netdoc_parseable_derive_debug(ttype: &str, msg: &str, vals: &[&dyn Debug]) {
+    // Take a lock like this so that all our output appears at once,
+    // rather than possibly being interleaved with similar output for other types.
     let mut out = std::io::stderr().lock();
     (|| {
         write!(out, "netdoc {ttype} parse: {msg}")?;
@@ -47,12 +15,223 @@ pub fn netdoc_parseable_derive_debug(ttype: &str, msg: &str, vals: &[&dyn Debug]
         }
         writeln!(out)
     })()
-    .expect("write to String failed");
+    .expect("write to stderr failed");
+}
+
+define_derive_deftly_module! {
+    /// Common definitions for `NetdocParseable` and `NetdocParseableFields`
+    ///
+    ///  * **`THIS_ITEM`**: consumes the next item and evaluates to it as an `UnparsedItem`.
+    ///    See the definition in `NetdocParseable`.
+    ///
+    ///  * **`F_ACCUMULATE_VAR`** the variable or field into which to accumulate
+    ///    normal items for this field.  Must be of type `&mut $F_ACCUMULATE_TYPE`.
+    ///
+    /// Importer must also import `NetdocSomeItemsDeriveCommon` and `NetdocDeriveAnyCommon`.
+    NetdocSomeItemsParseableCommon beta_deftly:
+
+    // Convenience alias for our prelude
+    ${define P { $crate::parse2::internal_prelude }}
+
+    // Defines the `dtrace` macro.
+    ${define DEFINE_DTRACE {
+        #[allow(unused_macros)]
+        macro_rules! dtrace { { $$msg:literal $$(, $$val:expr )* $$(,)? } => {
+          ${if tmeta(netdoc(debug)) {
+              $P::netdoc_parseable_derive_debug(
+                  ${concat $ttype},
+                  $$msg,
+                  &[ $$( &&$$val as _, )* ],
+              )
+          }}
+        }}
+    }}
+
+    // The effective field type for parsing.
+    //
+    // Handles #[deftly(netdoc(default))], in which case we parse as if the field was Option,
+    // and substitute in the default at the end.
+    //
+    ${define F_EFFECTIVE_TYPE {
+        ${if all(fmeta(netdoc(default))) {
+            Option::<$ftype>
+        } else {
+            $ftype
+        }}
+    }}
+
+    // Provide `$<selector_ $fname>` for every (suitable) field.
+    ${define ITEM_SET_SELECTORS {
+        $(
+          ${when not(any(F_FLATTEN))}
+
+          // See `mod multiplicity`.
+        ${if not(all(F_INTRO, fmeta(netdoc(with)))) {
+          // If the intro it has `with`, we don't check its trait impl, and this ends up unused
+          let $<selector_ $fname> = MultiplicitySelector::<$F_EFFECTIVE_TYPE>::default();
+        }}
+        )
+    }}
+    // The item set selector for this field.
+    // We must provide this, rather than expanding $<selector_ $fname> at the use sites,
+    // because the identifier `selector_` has different macro_rules hygiene here vs there!
+    // TODO derive-deftly#130
+    ${define F_SELECTOR $<selector_ $fname>}
+
+    // Check that every field type implements the necessary trait.
+    ${define CHECK_FIELD_TYPES_PARSEABLE {
+        $(
+          ${when not(any(F_FLATTEN))}
+
+          // Expands to `selector_FIELD.check_SOMETHING();`
+          //
+          // If the relevant trait isn't implemented, rustc reports the error by
+          // pointing at the `check-something` call.  We re-span that identifier
+          // to point to the field name, so that's where the error is reported.
+          //
+          // Without this, we just get a report that `item` doesn't implement the required
+          // trait - but `item` is a local variable here, so the error points into the macro
+        ${if not(all(any(F_INTRO, F_NORMAL), fmeta(netdoc(with)))) {
+          $<selector_ $fname> . ${paste_spanned $fname ${select1
+                  any(F_INTRO, F_NORMAL){
+                      // For the intro item, this is not completely precise, because the
+                      // it will allow Option<> and Vec<> which aren't allowed there.
+                      ${if
+                        fmeta(netdoc(single_arg)) { check_item_argument_parseable }
+                        else { check_item_value_parseable }
+                      }
+                  }
+                  F_SIGNATURE { check_signature_item_parseable }
+                  F_SUBDOC    { check_subdoc_parseable         }
+          }} ();
+        }}
+        )
+    }}
+
+    // Convert the UnparsedItem (in `item` to the value (to accumulate).
+    // Expands to an expression.
+    ${define ITEM_VALUE_FROM_UNPARSED {
+        ${if fmeta(netdoc(with)) {
+          ${fmeta(netdoc(with)) as path}
+              ::${paste_spanned $fname from_unparsed}
+              (item)?
+        } else if fmeta(netdoc(single_arg)) { {
+          let item = ItemValueParseable::from_unparsed(item)?;
+          let (item,) = item;
+          item
+        } } else {
+          ItemValueParseable::from_unparsed(item)?
+        }}
+    }}
+
+    // Type into which we accumulate value(s) of this field
+    ${define F_ACCUMULATE_TYPE {
+        ${if F_FLATTEN {
+            <$ftype as $P::NetdocParseableFields>::Accumulator
+        } else {
+            Option::<$F_EFFECTIVE_TYPE>
+        }
+    }}}
+
+    // Accumulates `item` (which must be `ItemSetMethods::Each`) into `$F_ACCUMULATE_VAR`
+    ${define ACCUMULATE_ITEM_VALUE { {
+        $F_SELECTOR.${paste_spanned $fname accumulate}($F_ACCUMULATE_VAR, item)?;
+    } }}
+
+    // Handle a nonstructural field, parsing and accumulating its value
+    //
+    // Looks at `kw` for the keyword.
+    //
+    // Expands to a series of `if ... { ... } else`.
+    // The use site must provide (maybe further arms) and a fallback block!
+    //
+    // If the item is the intro item for this document, evaluates `break` -
+    // so if `f_INTRO` is not trivially false, must be expanded within a field loop.
+    ${define NONSTRUCTURAL_ACCUMULATE_ELSE {
+        ${for fields {
+          ${when not(any(F_FLATTEN, F_SUBDOC))}
+
+          if kw == $F_KEYWORD {
+            ${select1
+              F_NORMAL {
+                let item = $THIS_ITEM;
+                dtrace!("is normal", item);
+                let item = $ITEM_VALUE_FROM_UNPARSED;
+                $ACCUMULATE_ITEM_VALUE
+              }
+              F_SIGNATURE {
+                let hash_inputs = input
+                      .peek_signature_hash_inputs(signed_doc_body)?
+                      .expect("not eof, we peeked kw");
+
+                let item = $THIS_ITEM;
+                dtrace!("is signature", item);
+                let item =
+                    SignatureItemParseable::from_unparsed_and_body(item, &hash_inputs)?;
+                $ACCUMULATE_ITEM_VALUE
+              }
+              F_INTRO {
+                dtrace!("is intro", kw);
+                break;
+              } // start of next similar document
+            }
+          } else
+        }}
+        ${for fields {
+          ${when F_FLATTEN}
+
+          if $ftype::is_item_keyword(kw) {
+              dtrace!(${concat "is flatten in " $fname}, kw);
+              let item = $THIS_ITEM;
+              <$ftype as NetdocParseableFields>::accumulate_item($F_ACCUMULATE_VAR, item)?;
+          } else
+        }}
+    }}
+
+    // Completes a document
+    //
+    // The fields accumulated so far must be in `$fpatname` (as a value, not a ref,
+    // and therefore not in $F_ACCUMULATE_VAR).
+    //
+    // Expands to code which resolves the fields, and ends with `Ok(document value)`.
+    ${define FINISH_RESOLVE {
+        ${for fields {
+            ${select1
+              F_INTRO {}
+              any(F_NORMAL, F_SIGNATURE) {
+                  let $fpatname = $F_SELECTOR.finish($fpatname, $F_KEYWORD_REPORT)?;
+              }
+              F_FLATTEN {
+                  let $fpatname = <$ftype as NetdocParseableFields>::finish($fpatname)?;
+              }
+              F_SUBDOC {
+                  let $fpatname = $F_SELECTOR.finish_subdoc($fpatname)?;
+              }
+            }
+        }}
+        $(
+            ${when not(F_INTRO)}
+            // These conditions are mirrored in NetdocSomeItemsEncodableCommon,
+            // which is supposed to recognise netdoc(default) precisely when we do.
+          ${if fmeta(netdoc(default)) {
+            let $fpatname = Option::unwrap_or_default($fpatname);
+          }}
+        )
+        Ok($vpat)
+    }}
 }
 
 define_derive_deftly! {
+    use NetdocDeriveAnyCommon;
+    use NetdocEntireDeriveCommon;
+    use NetdocSomeItemsDeriveCommon;
+    use NetdocSomeItemsParseableCommon;
+
     /// Derive [`NetdocParseable`] for a document (or sub-document)
     ///
+    // NB there is very similar wording in the NetdocEncodable derive docs.
+    // If editing any of this derive's documentation, considering editing that too.
+    //
     /// ### Expected input structure
     ///
     /// Should be applied named-field struct, where each field is
@@ -67,6 +246,7 @@ define_derive_deftly! {
     /// Each field must be
     ///  * `impl `[`ItemValueParseable`] for an "exactly once" field,
     ///  * `Vec<T: ItemValueParseable>` for "zero or more", or
+    ///  * `BTreeSet<T: ItemValueParseable + Ord>`, or
     ///  * `Option<T: ItemValueParseable>` for "zero or one".
     ///
     /// We don't directly support "at least once":
@@ -175,7 +355,7 @@ define_derive_deftly! {
     /// use tor_netdoc::derive_deftly_template_NetdocParseable;
     /// use tor_netdoc::derive_deftly_template_NetdocSigned;
     /// use tor_netdoc::derive_deftly_template_ItemValueParseable;
-    /// use tor_netdoc::parse2::{parse_netdoc, VerifyFailed};
+    /// use tor_netdoc::parse2::{parse_netdoc, ParseInput, VerifyFailed};
     /// use tor_netdoc::parse2::{SignatureItemParseable, SignatureHashInputs};
     ///
     /// #[derive(Deftly, Debug, Clone)]
@@ -221,58 +401,14 @@ define_derive_deftly! {
     ///     }
     /// }
     ///
-    /// let doc: NdThingSigned = parse_netdoc(&doc_text, "<input>").unwrap();
+    /// let input = ParseInput::new(&doc_text, "<input>");
+    /// let doc: NdThingSigned = parse_netdoc(&input).unwrap();
     /// let doc = doc.verify_foolish_timeless().unwrap();
     /// assert_eq!(doc.value.0, "something");
     /// ```
     export NetdocParseable for struct, expect items, beta_deftly:
 
-    // Convenience alias for our prelude
-    ${define P { $crate::parse2::internal_prelude }}
-
-    // Predicate for the toplevel
-    ${defcond T_SIGNATURES tmeta(netdoc(signatures))}
-
-    // Predicates for the field kinds
-    ${defcond F_INTRO all(not(T_SIGNATURES), approx_equal($findex, 0))}
-    ${defcond F_FLATTEN fmeta(netdoc(flatten))}
-    ${defcond F_SUBDOC fmeta(netdoc(subdoc))}
-    ${defcond F_SIGNATURE T_SIGNATURES} // signatures section documents have only signature fields
-    ${defcond F_NORMAL not(any(F_SIGNATURE, F_INTRO, F_FLATTEN, F_SUBDOC))}
-
-    // Field keyword as `&str`
-    ${define F_KEYWORD_STR { ${concat
-        ${if any(F_FLATTEN, F_SUBDOC) {
-          ${if F_INTRO {
-            ${error "#[deftly(netdoc(subdoc))] and (flatten) not supported for intro items"}
-          } else {
-            // Sub-documents and flattened fields have their keywords inside;
-            // if we ask for the field-based keyword name for one of those then that's a bug.
-            ${error "internal error, subdoc KeywordRef"}
-          }}
-        }}
-        ${fmeta(netdoc(keyword)) as str,
-          default ${concat ${kebab_case $fname}}}
-    }}}
-    // Field keyword as `&str` for debugging and error reporting
-    ${define F_KEYWORD_REPORT {
-        ${if F_SUBDOC { ${concat $fname} }
-             else { $F_KEYWORD_STR }}
-    }}
-    // Field keyword as `KeywordRef`
-    ${define F_KEYWORD { (KeywordRef::new_const($F_KEYWORD_STR)) }}
-
-    // The effective field type for parsing.
-    //
-    // Handles #[deftly(netdoc(default))], in which case we parse as if the field was Option,
-    // and substitute in the default at the end.
-    ${define F_EFFECTIVE_TYPE {
-        ${if all(fmeta(netdoc(default)), not(F_INTRO)) {
-            Option::<$ftype>
-        } else {
-            $ftype
-        }}
-    }}
+    ${define F_ACCUMULATE_VAR { (&mut $fpatname) }}
 
     impl<$tgens> $P::NetdocParseable for $ttype {
         fn doctype_for_error() -> &'static str {
@@ -291,74 +427,18 @@ define_derive_deftly! {
 
         //##### main parsing function #####
 
+        #[allow(clippy::redundant_locals)] // let item = $THIS_ITEM, which might be item
         fn from_items<'s>(
             input: &mut $P::ItemStream<'s>,
             outer_stop: $P::stop_at!(),
         ) -> $P::Result<$ttype, $P::ErrorProblem> {
             use $P::*;
-
-            //----- compile-time check that fields are in the right order in the struct -----
-
-            ${if not(T_SIGNATURES) { // signatures structs have only signature fields
-              netdoc_ordering_check! {
-                $(
-                    ${select1
-                      F_INTRO     { intro     }
-                      F_NORMAL    { normal    }
-                      F_FLATTEN   { normal    }
-                      F_SUBDOC    { subdoc    }
-                    }
-                    $fname
-                )
-              }
-            }}
-
-            //----- Debugging -----
-
-            macro_rules! dtrace { { $$msg:literal $$(, $$val:expr )* $$(,)? } => {
-              ${if tmeta(netdoc(debug)) {
-                  netdoc_parseable_derive_debug(
-                      ${concat $ttype},
-                      $$msg,
-                      &[ $$( &&$$val as _, )* ],
-                  )
-              }}
-            }}
+            $DEFINE_DTRACE
+            $FIELD_ORDERING_CHECK
 
             //----- prepare item set selectors for every field -----
-
-          $(
-            ${when not(any(F_FLATTEN))}
-
-            // See `mod multiplicity`.
-          ${if not(all(F_INTRO, fmeta(netdoc(with)))) {
-            // If the intro it has `with`, we don't check its trait impl, and this ends up unused
-            let $<selector_ $fname> = ItemSetSelector::<$F_EFFECTIVE_TYPE>::default();
-          }}
-
-            // Expands to `selector_FIELD.check_SOMETHING();`
-            //
-            // If the relevant trait isn't implemented, rustc reports the error by
-            // pointing at the `check-something` call.  We re-span that identifier
-            // to point to the field name, so that's where the error is reported.
-            //
-            // Without this, we just get a report that `item` doesn't implement the required
-            // trait - but `item` is a local variable here, so the error points into the macro
-          ${if not(all(any(F_INTRO, F_NORMAL), fmeta(netdoc(with)))) {
-            $<selector_ $fname> . ${paste_spanned $fname ${select1
-                    any(F_INTRO, F_NORMAL){
-                        // For the intro item, this is not completely precise, because the
-                        // it will allow Option<> and Vec<> which aren't allowed there.
-                        ${if
-                          fmeta(netdoc(single_arg)) { check_item_argument_parseable }
-                          else { check_item_value_parseable }
-                        }
-                    }
-                    F_SIGNATURE { check_signature_item_parseable }
-                    F_SUBDOC    { check_subdoc_parseable         }
-            }} ();
-          }}
-          )
+            $ITEM_SET_SELECTORS
+            $CHECK_FIELD_TYPES_PARSEABLE
 
             // Is this an intro item keyword ?
             //
@@ -367,7 +447,7 @@ define_derive_deftly! {
             //    fn(KeywordRef) -> bool
             ${define F_SUBDOC_IS_INTRO_ITEM_KEYWORD {
                 ${if not(F_SUBDOC) { ${error "internal-error: subdoc kw, but not subdoc field"} }}
-                $<selector_ $fname>.is_intro_item_keyword
+                $F_SELECTOR.is_intro_item_keyword
             }}
 
             //----- Helper fragments for parsing individual pieces of the document -----
@@ -392,25 +472,6 @@ define_derive_deftly! {
                 input.next_item()?.expect("peeked")
             }}
 
-            ${define ITEM_VALUE_FROM_UNPARSED {
-              ${if fmeta(netdoc(with)) {
-                ${fmeta(netdoc(with)) as path}
-                    ::${paste_spanned $fname from_unparsed}
-                    (item)?
-              } else if fmeta(netdoc(single_arg)) { {
-                let item = ItemValueParseable::from_unparsed(item)?;
-                let (item,) = item;
-                item
-              } } else {
-                ItemValueParseable::from_unparsed(item)?
-              }}
-            }}
-
-            // Accumulates `item` (which must be DataSet::Value) into `Putnam`
-            ${define ACCUMULATE_ITEM_VALUE { {
-                $<selector_ $fname>.accumulate(&mut $fpatname, item)?;
-            } }}
-
             //----- keyword classification closures -----
 
             // Is this a keyword for one of our sub-documents?
@@ -451,13 +512,9 @@ define_derive_deftly! {
             }
             let $fpatname: $ftype = $ITEM_VALUE_FROM_UNPARSED;
 
-          } F_FLATTEN {
-
-            let mut $fpatname = <$ftype as NetdocParseableFields>::Accumulator::default();
-
           } else {
 
-            let mut $fpatname: Option<$F_EFFECTIVE_TYPE> = None;
+            let mut $fpatname = $F_ACCUMULATE_TYPE::default();
 
           }})
 
@@ -470,44 +527,8 @@ define_derive_deftly! {
                     dtrace!("is inner stop", kw);
                     break;
                 };
-              ${for fields {
-                ${when not(any(F_FLATTEN, F_SUBDOC))}
 
-                if kw == $F_KEYWORD {
-                  ${select1
-                    F_NORMAL {
-                      let item = $THIS_ITEM;
-                      dtrace!("is normal", item);
-                      let item = $ITEM_VALUE_FROM_UNPARSED;
-                      $ACCUMULATE_ITEM_VALUE
-                    }
-                    F_SIGNATURE {
-                      let hash_inputs = input
-                            .peek_signature_hash_inputs(signed_doc_body)?
-                            .expect("not eof, we peeked kw");
-
-                      let item = $THIS_ITEM;
-                      dtrace!("is signature", item);
-                      let item =
-                          SignatureItemParseable::from_unparsed_and_body(item, &hash_inputs)?;
-                      $ACCUMULATE_ITEM_VALUE
-                    }
-                    F_INTRO {
-                      dtrace!("is intro", kw);
-                      break;
-                    } // start of next similar document
-                  }
-                } else
-              }}
-              ${for fields {
-                ${when F_FLATTEN}
-
-                if $ftype::is_item_keyword(kw) {
-                    dtrace!(${concat "is flatten in " $fname}, kw);
-                    let item = $THIS_ITEM;
-                    <$ftype as NetdocParseableFields>::accumulate_item(&mut $fpatname, item)?;
-                } else
-              }}
+                $NONSTRUCTURAL_ACCUMULATE_ELSE
                 {
                     dtrace!("is unknown (in normal)");
                     let _: UnparsedItem = $THIS_ITEM;
@@ -530,7 +551,7 @@ define_derive_deftly! {
                     break;
                 };
 
-                $<selector_ $fname>.can_accumulate(&mut $fpatname)?;
+                $F_SELECTOR.can_accumulate(&mut $fpatname)?;
 
                 dtrace!("is this subdoc", kw);
                 let item = NetdocParseable::from_items(input, inner_stop);
@@ -544,34 +565,17 @@ define_derive_deftly! {
             // Resolve all the fields
             dtrace!("reached end, resolving");
 
-          ${for fields {
-            ${select1
-              F_INTRO {}
-              any(F_NORMAL, F_SIGNATURE) {
-                  let $fpatname = $<selector_ $fname>.finish($fpatname, $F_KEYWORD_REPORT)?;
-              }
-              F_FLATTEN {
-                  let $fpatname = <$ftype as NetdocParseableFields>::finish($fpatname)?;
-              }
-              F_SUBDOC {
-                  let $fpatname = $<selector_ $fname>.finish_subdoc($fpatname)?;
-              }
-          }}}
-          $(
-            ${when not(F_INTRO)}
-          ${if fmeta(netdoc(default)) {
-            let $fpatname = Option::unwrap_or_default($fpatname);
-          }}
-          )
-
-            let r = $vpat;
-
-            Ok(r)
+            $FINISH_RESOLVE
         }
     }
 }
 
 define_derive_deftly! {
+    use NetdocDeriveAnyCommon;
+    use NetdocFieldsDeriveCommon;
+    use NetdocSomeItemsDeriveCommon;
+    use NetdocSomeItemsParseableCommon;
+
     /// Derive [`NetdocParseableFields`] for a struct with individual items
     ///
     /// Defines a struct `FooNetdocParseAccumulator` to be the
@@ -582,60 +586,20 @@ define_derive_deftly! {
     /// but:
     ///
     ///  * Derives [`NetdocParseableFields`]
-    ///  * The input struct can contain only normal non-structural items
-    ///    (so it's not a sub-document with an intro item).
-    ///  * The only attributes supported are the field attributes
-    ///    `#[deftly(netdoc(keyword = STR))]`
-    ///    `#[deftly(netdoc(default))]`
-    ///    `#[deftly(netdoc(single_arg))]`
-    ///    `#[deftly(netdoc(with = "MODULE"))]`
-    ///    `#[deftly(netdoc(flatten))]`
+    $DOC_NETDOC_FIELDS_DERIVE_SUPPORTED
+    ///
     export NetdocParseableFields for struct , expect items, beta_deftly:
 
-    // TODO deduplicate with copy in NetdocParseable after rust-derive-deftly#39
-
-    // Convenience alias for our prelude
-    ${define P { $crate::parse2::internal_prelude }}
-
-    // The effective field type for parsing.
-    //
-    // Handles #[deftly(netdoc(default))], in which case we parse as if the field was Option,
-    // and substitute in the default at the end.
-    //
-    ${define F_EFFECTIVE_TYPE {
-        ${if all(fmeta(netdoc(default))) {
-            Option::<$ftype>
-        } else {
-            $ftype
-        }}
-    }}
-    ${define F_ITEM_SET_SELECTOR {
-        ItemSetSelector::<$F_EFFECTIVE_TYPE>::default()
-    }}
-    ${defcond F_FLATTEN fmeta(netdoc(flatten))}
-
-    // NOTE! These keyword defines are simpler than the ones for NetdocParseable.
-    // Care must be taken if they are deduplicated as noted above.
-    // Field keyword as `&str`
-    ${define F_KEYWORD_STR { ${concat
-        ${fmeta(netdoc(keyword)) as str,
-          default ${kebab_case $fname}}
-    }}}
-    // Field keyword as `KeywordRef`
-    ${define F_KEYWORD { (KeywordRef::new_const($F_KEYWORD_STR)) }}
+    ${define THIS_ITEM item}
+    ${define F_ACCUMULATE_VAR { (&mut acc.$fname) }}
 
     #[doc = ${concat "Partially parsed `" $tname "`"}]
     ///
     /// Used for [`${concat $P::NetdocParseableFields::Accumulator}`].
     #[derive(Default, Debug)]
-    $tvis struct $<$tname NetdocParseAccumulator><$tdefgens> { $( ${select1
-      F_FLATTEN {
-        $fname: <$ftype as $P::NetdocParseableFields>::Accumulator,
-      }
-      else {
-        $fname: Option<$F_EFFECTIVE_TYPE>,
-      }
-    } ) }
+    $tvis struct $<$tname NetdocParseAccumulator><$tdefgens> { $(
+        $fname: $F_ACCUMULATE_TYPE,
+    ) }
 
     impl<$tgens> $P::NetdocParseableFields for $ttype {
         type Accumulator = $<$ttype NetdocParseAccumulator>;
@@ -658,6 +622,7 @@ define_derive_deftly! {
             false
         }
 
+        #[allow(clippy::redundant_locals)] // let item = $THIS_ITEM, which might be item
         fn accumulate_item(
             #[allow(unused_variables)] // If there are no fields, this is unused
             acc: &mut Self::Accumulator,
@@ -666,37 +631,21 @@ define_derive_deftly! {
         ) -> $P::Result<(), $P::ErrorProblem> {
             #[allow(unused_imports)] // false positives in some situations
             use $P::*;
+            $DEFINE_DTRACE
+
+            $ITEM_SET_SELECTORS
+            $CHECK_FIELD_TYPES_PARSEABLE
 
             #[allow(unused_variables)] // If there are no fields, this is unused
             let kw = item.keyword();
-          $(
-            ${when not(F_FLATTEN)}
-            if kw == $F_KEYWORD {
-                let selector = $F_ITEM_SET_SELECTOR;
-              ${if fmeta(netdoc(with)) {
-                let item = ${fmeta(netdoc(with)) as path}
-                    ::${paste_spanned $fname from_unparsed}
-                    (item)?;
-              } else if fmeta(netdoc(single_arg)) {
-                selector.${paste_spanned $fname check_item_argument_parseable}();
-                let item = ItemValueParseable::from_unparsed(item)?;
-                let (item,) = item;
-              } else {
-                selector.${paste_spanned $fname check_item_value_parseable}();
-                let item = ItemValueParseable::from_unparsed(item)?;
-              }}
-                selector.accumulate(&mut acc.$fname, item)
-            } else
-          )
-          $(
-            ${when F_FLATTEN}
-            if <$ftype as NetdocParseableFields>::is_item_keyword(kw) {
-                <$ftype as NetdocParseableFields>::accumulate_item(&mut acc.$fname, item)
-            } else
-          )
+
+            $NONSTRUCTURAL_ACCUMULATE_ELSE
             {
                 panic!("accumulate_item called though is_intro_item_keyword returns false");
             }
+
+            #[allow(unreachable_code)] // If there are no fields!
+            Ok(())
         }
 
         fn finish(
@@ -705,24 +654,21 @@ define_derive_deftly! {
         ) -> $P::Result<Self, $P::ErrorProblem> {
             #[allow(unused_imports)] // false positives in some situations
             use $P::*;
+            $DEFINE_DTRACE
 
-          $(
-            ${when not(F_FLATTEN)}
-            let $fpatname = $F_ITEM_SET_SELECTOR.finish(acc.$fname, $F_KEYWORD_STR)?;
-          ${if fmeta(netdoc(default)) {
-            let $fpatname = Option::unwrap_or_default($fpatname);
-          }}
-          )
-          $(
-            ${when F_FLATTEN}
-            let $fpatname = <$ftype as NetdocParseableFields>::finish(acc.$fname)?;
-          )
-            Ok($vpat)
+            dtrace!("finish, resolving");
+
+            $ITEM_SET_SELECTORS
+
+         $( let $fpatname = acc.$fname; )
+            $FINISH_RESOLVE
         }
     }
 }
 
 define_derive_deftly! {
+    use NetdocDeriveAnyCommon;
+
     /// Derive `FooSigned` from `Foo`
     ///
     /// Apply this derive to the main body struct `Foo`.
@@ -741,6 +687,8 @@ define_derive_deftly! {
     ///   Normally this is achieved with
     ///   `#[derive_deftly(NetdocParseable)] #[deftly(netdoc(signatures))]`.
     ///
+    $DOC_DEBUG_PLACEHOLDER
+    ///
     /// ### Generated struct
     ///
     /// ```
@@ -802,6 +750,7 @@ define_derive_deftly! {
             input: &mut $P::ItemStream<'s>,
             outer_stop: $P::stop_at!(),
         ) -> $P::Result<$<$ttype Signed>, $P::ErrorProblem> {
+            $EMIT_DEBUG_PLACEHOLDER
             input.parse_signed(outer_stop)
         }
     }
@@ -822,8 +771,14 @@ define_derive_deftly! {
 }
 
 define_derive_deftly! {
+    use NetdocDeriveAnyCommon;
+    use NetdocItemDeriveCommon;
+
     /// Derive `ItemValueParseable`
     ///
+    // NB there is very similar wording in the ItemValueEncodable derive docs.
+    // If editing any of this derive's documentation, considering editing that too.
+    //
     /// Fields in the struct are parsed from the keyword line arguments,
     /// in the order they appear in the struct.
     ///
@@ -832,12 +787,12 @@ define_derive_deftly! {
     /// Each field should be:
     ///
     ///  * `impl `[`ItemArgumentParseable`] (one argument),
-    ///  * `Option<impl ItemArgumentParseable>` (one optional argument), or
-    ///  * `Vec<impl ItemArgumentParseable>` (zero or more arguments).
+    ///  * `Option<impl ItemArgumentParseable>` (one optional argument),
+    ///  * `Vec<impl ItemArgumentParseable>` (zero or more arguments), or
+    ///  * `BTreeSet<impl ItemArgumentParseable + Ord>` (zero or more arguments).
     ///
-    /// `ItemArgumentParseable` is implemented for every `impl FromStr`,
-    /// so `impl FromStr`, `Option<impl FromStr>` and `Vec<impl FromStr>`
-    /// are supported.
+    /// `ItemArgumentParseable` can be implemented via `impl FromStr`,
+    /// by writing `impl NormalItemArgument`.
     ///
     /// For `Option` or `Vec`, we expect that *if* there are any further arguments,
     /// they are for this field.
@@ -858,14 +813,26 @@ define_derive_deftly! {
     ///    Reject, rather than ignore, additional arguments found in the document
     ///    which aren't described by the struct.
     ///
+    /// * **`#[deftly(netdoc(debug))]`**:
+    ///
+    ///   Currently implemented only as a placeholde
+    ///
+    ///   The generated implementation may in future generate copious debug output
+    ///   to the program's stderr when it is run.
+    ///   Do not enable in production!
+    ///
+    $DOC_DEBUG_PLACEHOLDER
+    ///
     /// ### Field-level attributes:
     ///
     ///  * **`#[deftly(netdoc(rest))]**:
     ///
     ///    The field is the whole rest of the line.
     ///    Must come after any other normal argument fields.
+    ///    Only allowed once.
     ///
     ///    The field type must implement `FromStr`.
+    ///    (I.e. `Vec` , `Option` etc., are not allowed, and `ItemArgumentParseable` is not used.)
     ///
     ///  * **`#[deftly(netdoc(object))]**:
     ///
@@ -883,18 +850,20 @@ define_derive_deftly! {
     ///
     ///  * **`#[deftly(netdoc(with = "MODULE")]**:
     ///
-    ///    Instead of `ItemArgumentParseable`, the item is parsed with `MODULE::from_args`,
+    ///    Instead of `ItemArgumentParseable`, the argument is parsed with `MODULE::from_args`,
     ///    which must have the same signature as [`ItemArgumentParseable::from_args`].
     ///
-    ///    With `#[deftly(netdoc(rest))]`, FUNCTION replaces
-    ///    `<FIELD AS FromStr>::from_str`.
+    ///    With `#[deftly(netdoc(rest))]`, the argument is parsed with `MODULE::from_args_rest`,
+    ///    must have the signature
+    ///    `fn from_args_rest(s: &str) -> Result<FIELD, _>`).
+    ///    and replaces `<FIELD as FromStr>::from_str`.
     ///
-    ///    With `#[deftly(netdoc(objecte))]`, uses `MODULE::try_from`
-    ///    must have the signature `fn(Vec<u8>) -> Result<OBJECT, _>;
+    ///    With `#[deftly(netdoc(object))]`, uses `MODULE::try_from`
+    ///    which must have the signature `fn(Vec<u8>) -> Result<OBJECT, _>;
     ///    like `TryFrom::<Vec<u8>>>::try_from`.
     ///    LABEL must also be specified
     ///    unless the object also implements `ItemObjectParseable`.
-    ///    Errors from parsing will be discarded and replaced with
+    ///    Errors from parsing will all be collapsed into
     ///    [`ErrorProblem::ObjectInvalidData`].
     ///
     ///  * **`#[deftly(netdoc(sig_hash = "HASH_METHOD"))]**:
@@ -911,12 +880,6 @@ define_derive_deftly! {
 
     ${define P { $crate::parse2::internal_prelude }}
 
-    ${defcond F_REST fmeta(netdoc(rest))}
-    ${defcond F_OBJECT fmeta(netdoc(object))}
-    ${defcond F_SIG_HASH fmeta(netdoc(sig_hash))}
-    ${defcond F_NORMAL not(any(F_REST, F_OBJECT, F_SIG_HASH))}
-
-    ${defcond T_IS_SIGNATURE not(approx_equal(${for fields { ${when F_SIG_HASH} 1 }}, {}))}
     ${define TRAIT ${if T_IS_SIGNATURE { SignatureItemParseable } else { ItemValueParseable }}}
     ${define METHOD ${if T_IS_SIGNATURE { from_unparsed_and_body } else { from_unparsed }}}
 
@@ -931,13 +894,15 @@ define_derive_deftly! {
             #[allow(unused_imports)] // false positive when macro is used with prelude in scope
             use $P::*;
 
+            $EMIT_DEBUG_PLACEHOLDER
+
             let object = input.object();
             #[allow(unused)]
             let mut args = input.args_mut();
           $(
             let $fpatname = ${select1
               F_NORMAL { {
-                  let selector = ArgumentSetSelector::<$ftype>::default();
+                  let selector = MultiplicitySelector::<$ftype>::default();
                 ${if not(fmeta(netdoc(with))) {
                   selector.${paste_spanned $fname check_argument_value_parseable}();
                 }}
@@ -949,7 +914,7 @@ define_derive_deftly! {
                   ).map_err(args.error_handler(stringify!($fname)))?
               } }
               F_OBJECT { {
-                  let selector = ObjectSetSelector::<$ftype>::default();
+                  let selector = MultiplicitySelector::<$ftype>::default();
                   let object = object.map(|object| {
                       let data = object.decode_data()?;
                       ${if fmeta(netdoc(object(label))) {
@@ -974,9 +939,12 @@ define_derive_deftly! {
                   // consumes `args`, leading to compile error if the rest field
                   // isn't last (or is combined with no_extra_args).
                   let args_consume = args;
-                  ${fmeta(netdoc(with))
-                    as path,
-                    default { <$ftype as FromStr>::from_str }}(args_consume.into_remaining())
+                  ${if fmeta(netdoc(with)) {
+                      ${fmeta(netdoc(with)) as path}::${paste_spanned $fname from_args_rest}
+                  } else {
+                      <$ftype as FromStr>::from_str
+                  }}
+                      (args_consume.into_remaining())
                       .map_err(|_| AE::Invalid)
                       .map_err(args_consume.error_handler(stringify!($fname)))?
               } }
