@@ -3,13 +3,13 @@
 // NOTE: This is a work in progress and I bet I'll refactor it a lot;
 // it needs to stay opaque!
 
+use crate::circuit::CircuitRxSender;
 use crate::client::circuit::padding::{PaddingController, QueuedCellPaddingInfo};
 use crate::{Error, Result};
 use tor_basic_utils::RngExt;
 use tor_cell::chancell::CircId;
 
 use crate::circuit::celltypes::CreateResponse;
-use crate::client::circuit::CircuitRxSender;
 use crate::client::circuit::halfcirc::HalfCirc;
 
 use oneshot_fused_workaround as oneshot;
@@ -68,7 +68,7 @@ pub(super) enum CircEnt {
         /// A sink which should receive all the relay cells for this circuit
         /// from this channel
         cell_sender: CircuitRxSender,
-        //// A padding controller we should use when reporting flushed cells.
+        /// A padding controller we should use when reporting flushed cells.
         padding_ctrl: PaddingController,
     },
 
@@ -77,7 +77,7 @@ pub(super) enum CircEnt {
         /// A sink which should receive all the relay cells for this circuit
         /// from this channel
         cell_sender: CircuitRxSender,
-        //// A padding controller we should use when reporting flushed cells.
+        /// A padding controller we should use when reporting flushed cells.
         padding_ctrl: PaddingController,
     },
 
@@ -283,7 +283,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
