@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -56,28 +56,44 @@
 #[cfg(feature = "bench")]
 pub mod bench_utils;
 pub mod channel;
+pub mod circuit;
+pub mod client;
+pub(crate) mod conflux;
 mod congestion;
 mod crypto;
 pub mod memquota;
-pub mod stream;
+mod stream;
+pub(crate) mod streammap;
 pub(crate) mod tunnel;
 mod util;
 
+#[cfg(feature = "relay")]
+pub mod relay;
+#[cfg(feature = "relay")]
+pub use relay::channel::{RelayChannelBuilder, RelayIdentities};
+
 pub use util::err::{Error, ResolveError};
 pub use util::skew::ClockSkew;
 
 pub use channel::params::ChannelPaddingInstructions;
+pub use client::{ClientTunnel, HopLocation, TargetHop};
 pub use congestion::params as ccparams;
 pub use crypto::cell::{HopNum, HopNumDisplay};
-pub use tunnel::{circuit, HopLocation, TargetHop};
+pub use stream::flow_ctrl::params::{CellCount, FlowCtrlParameters};
+#[cfg(feature = "send-control-msg")]
+pub use {
+    crate::client::Conversation,
+    crate::client::msghandler::{MsgHandler, UserMsgHandler},
+    crate::client::reactor::MetaCellDisposition,
+};
 
 /// A Result type for this crate.
 pub type Result<T> = std::result::Result<T, Error>;
 
 use std::fmt::Debug;
 use tor_memquota::{
-    mq_queue::{self, ChannelSpec as _},
     HasMemoryCost,
+    mq_queue::{self, ChannelSpec as _},
 };
 use tor_rtcompat::DynTimeProvider;
 
@@ -139,7 +155,7 @@ pub fn supported_client_protocols() -> tor_protover::Protocols {
     use tor_protover::named::*;
     // WARNING: REMOVING ELEMENTS FROM THIS LIST CAN BE DANGEROUS!
     // SEE [`tor_protover::doc_changing`]
-    [
+    let mut protocols = vec![
         LINK_V4,
         LINK_V5,
         LINKAUTH_ED25519_SHA256_EXPORTER,
@@ -147,9 +163,14 @@ pub fn supported_client_protocols() -> tor_protover::Protocols {
         RELAY_NTOR,
         RELAY_EXTEND_IPv6,
         RELAY_NTORV3,
-    ]
-    .into_iter()
-    .collect()
+        RELAY_NEGOTIATE_SUBPROTO,
+    ];
+    #[cfg(feature = "flowctl-cc")]
+    protocols.push(FLOWCTRL_CC);
+    #[cfg(feature = "counter-galois-onion")]
+    protocols.push(RELAY_CRYPT_CGO);
+
+    protocols.into_iter().collect()
 }
 
 #[cfg(test)]
@@ -163,17 +184,28 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
+    use cfg_if::cfg_if;
+
     use super::*;
 
     #[test]
     fn protocols() {
         let pr = supported_client_protocols();
-        let expected = "FlowCtrl=1 Link=4-5 LinkAuth=3 Relay=2-4".parse().unwrap();
+        cfg_if! {
+            if #[cfg(all(feature="flowctl-cc", feature="counter-galois-onion"))] {
+                let expected = "FlowCtrl=1-2 Link=4-5 LinkAuth=3 Relay=2-6".parse().unwrap();
+            } else if #[cfg(feature="flowctl-cc")] {
+                let expected = "FlowCtrl=1-2 Link=4-5 LinkAuth=3 Relay=2-5".parse().unwrap();
+                // (Note that we don't have to check for cgo without cc, since that isn't possible.)
+            } else {
+                let expected = "FlowCtrl=1 Link=4-5 LinkAuth=3 Relay=2-5".parse().unwrap();
+            }
+        }
         assert_eq!(pr, expected);
     }
 }
