@@ -1,21 +1,78 @@
-//! Functionality for incoming streams, opened from the other side of a circuit.
+//! Incoming data stream cell handlers, shared by the relay and onion service implementations.
 
 use bitvec::prelude::*;
-
-use super::{AnyCmdChecker, DataStream, StreamReceiver, StreamStatus};
-use crate::circuit::ClientCircSyncView;
-use crate::memquota::StreamAccount;
-use crate::tunnel::reactor::CloseStreamBehavior;
-use crate::tunnel::StreamTarget;
-use crate::{Error, Result};
 use derive_deftly::Deftly;
 use oneshot_fused_workaround as oneshot;
-use tor_cell::relaycell::{msg, RelayCmd, UnparsedRelayMsg};
+use postage::watch;
+
+use tor_cell::relaycell::msg::AnyRelayMsg;
+use tor_cell::relaycell::{RelayCellFormat, RelayCmd, StreamId, UnparsedRelayMsg, msg};
 use tor_cell::restricted_msg;
 use tor_error::internal;
 use tor_memquota::derive_deftly_template_HasMemoryCost;
 use tor_rtcompat::DynTimeProvider;
 
+use crate::circuit::CircSyncView;
+use crate::stream::cmdcheck::{AnyCmdChecker, CmdChecker, StreamStatus};
+use crate::stream::{CloseStreamBehavior, StreamComponents};
+use crate::{Error, Result};
+
+// TODO(relay): move these to a shared module
+use crate::HopLocation;
+use crate::client::stream::DataStream;
+use crate::memquota::StreamAccount;
+use crate::stream::StreamMpscSender;
+use crate::stream::flow_ctrl::state::StreamRateLimit;
+use crate::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
+use crate::stream::queue::StreamQueueReceiver;
+use crate::util::notify::NotifyReceiver;
+
+use std::mem::size_of;
+
+/// A `CmdChecker` that enforces invariants for inbound data streams.
+#[derive(Debug, Default)]
+pub(crate) struct InboundDataCmdChecker;
+
+restricted_msg! {
+    /// An allowable incoming message on an incoming data stream.
+    enum IncomingDataStreamMsg:RelayMsg {
+        // SENDME is handled by the reactor.
+        Data, End,
+    }
+}
+
+impl CmdChecker for InboundDataCmdChecker {
+    fn check_msg(&mut self, msg: &tor_cell::relaycell::UnparsedRelayMsg) -> Result<StreamStatus> {
+        use StreamStatus::*;
+        match msg.cmd() {
+            RelayCmd::DATA => Ok(Open),
+            RelayCmd::END => Ok(Closed),
+            _ => Err(Error::StreamProto(format!(
+                "Unexpected {} on an incoming data stream!",
+                msg.cmd()
+            ))),
+        }
+    }
+
+    fn consume_checked_msg(&mut self, msg: tor_cell::relaycell::UnparsedRelayMsg) -> Result<()> {
+        let _ = msg
+            .decode::<IncomingDataStreamMsg>()
+            .map_err(|err| Error::from_bytes_err(err, "cell on half-closed stream"))?;
+        Ok(())
+    }
+}
+
+impl InboundDataCmdChecker {
+    /// Return a new boxed `DataCmdChecker` in a state suitable for a
+    /// connection where an initial CONNECTED cell is not expected.
+    ///
+    /// This is used by hidden services, exit relays, and directory servers
+    /// to accept streams.
+    pub(crate) fn new_connected() -> AnyCmdChecker {
+        Box::new(Self)
+    }
+}
+
 /// A pending request from the other end of the circuit for us to open a new
 /// stream.
 ///
@@ -31,12 +88,8 @@ pub struct IncomingStream {
     time_provider: DynTimeProvider,
     /// The message that the client sent us to begin the stream.
     request: IncomingStreamRequest,
-    /// The information that we'll use to wire up the stream, if it is accepted.
-    stream: StreamTarget,
-    /// The underlying `StreamReceiver`.
-    receiver: StreamReceiver,
-    /// The memory quota account that should be used for this stream's data
-    memquota: StreamAccount,
+    /// Stream components used to assemble the [`DataStream`].
+    components: StreamComponents,
 }
 
 impl IncomingStream {
@@ -44,16 +97,12 @@ impl IncomingStream {
     pub(crate) fn new(
         time_provider: DynTimeProvider,
         request: IncomingStreamRequest,
-        stream: StreamTarget,
-        receiver: StreamReceiver,
-        memquota: StreamAccount,
+        components: StreamComponents,
     ) -> Self {
         Self {
             time_provider,
             request,
-            stream,
-            receiver,
-            memquota,
+            components,
         }
     }
 
@@ -68,18 +117,23 @@ impl IncomingStream {
         let Self {
             time_provider,
             request,
-            mut stream,
-            receiver,
-            memquota,
+            components:
+                StreamComponents {
+                    mut target,
+                    stream_receiver,
+                    xon_xoff_reader_ctrl,
+                    memquota,
+                },
         } = self;
 
         match request {
             IncomingStreamRequest::Begin(_) | IncomingStreamRequest::BeginDir(_) => {
-                stream.send(message.into()).await?;
+                target.send(message.into()).await?;
                 Ok(DataStream::new_connected(
                     time_provider,
-                    receiver,
-                    stream,
+                    stream_receiver,
+                    xon_xoff_reader_ctrl,
+                    target,
                     memquota,
                 ))
             }
@@ -103,7 +157,7 @@ impl IncomingStream {
         &mut self,
         message: CloseStreamBehavior,
     ) -> Result<oneshot::Receiver<Result<()>>> {
-        self.stream.close_pending(message)
+        self.components.target.close_pending(message)
     }
 
     /// Ignore this request without replying to the client.
@@ -171,7 +225,7 @@ impl IncomingCmdChecker {
     }
 }
 
-impl super::CmdChecker for IncomingCmdChecker {
+impl CmdChecker for IncomingCmdChecker {
     fn check_msg(&mut self, msg: &UnparsedRelayMsg) -> Result<StreamStatus> {
         if self.allow_commands[u8::from(msg.cmd()) as usize] {
             Ok(StreamStatus::Open)
@@ -205,7 +259,7 @@ pub trait IncomingStreamRequestFilter: Send + 'static {
     fn disposition(
         &mut self,
         ctx: &IncomingStreamRequestContext<'_>,
-        circ: &ClientCircSyncView<'_>,
+        circ: &CircSyncView<'_>,
     ) -> Result<IncomingStreamRequestDisposition>;
 }
 
@@ -227,7 +281,6 @@ pub struct IncomingStreamRequestContext<'a> {
     /// The request message itself
     pub(crate) request: &'a IncomingStreamRequest,
 }
-
 impl<'a> IncomingStreamRequestContext<'a> {
     /// Return a reference to the message used to request this stream.
     pub fn request(&self) -> &'a IncomingStreamRequest {
@@ -235,6 +288,44 @@ impl<'a> IncomingStreamRequestContext<'a> {
     }
 }
 
+/// Information about an incoming stream request.
+#[derive(Debug, Deftly)]
+#[derive_deftly(HasMemoryCost)]
+pub(crate) struct StreamReqInfo {
+    /// The [`IncomingStreamRequest`].
+    pub(crate) req: IncomingStreamRequest,
+    /// The ID of the stream being requested.
+    pub(crate) stream_id: StreamId,
+    /// The [`HopNum`](crate::HopNum).
+    ///
+    /// Set to `None` if we are an exit relay.
+    //
+    // TODO: For onion services, we might be able to enforce the HopNum earlier: we would never accept an
+    // incoming stream request from two separate hops.  (There is only one that's valid.)
+    pub(crate) hop: Option<HopLocation>,
+    /// The format which must be used with this stream to encode messages.
+    #[deftly(has_memory_cost(indirect_size = "0"))]
+    pub(crate) relay_cell_format: RelayCellFormat,
+    /// A channel for receiving messages from this stream.
+    #[deftly(has_memory_cost(indirect_size = "0"))] // estimate
+    pub(crate) receiver: StreamQueueReceiver,
+    /// A channel for sending messages to be sent on this stream.
+    #[deftly(has_memory_cost(indirect_size = "size_of::<AnyRelayMsg>()"))] // estimate
+    pub(crate) msg_tx: StreamMpscSender<AnyRelayMsg>,
+    /// A [`Stream`](futures::Stream) that provides updates to the rate limit for sending data.
+    // TODO(arti#2068): we should consider making this an `Option`
+    // the `watch::Sender` owns the indirect data
+    #[deftly(has_memory_cost(indirect_size = "0"))]
+    pub(crate) rate_limit_stream: watch::Receiver<StreamRateLimit>,
+    /// A [`Stream`](futures::Stream) that provides notifications when a new drain rate is
+    /// requested.
+    #[deftly(has_memory_cost(indirect_size = "0"))]
+    pub(crate) drain_rate_request_stream: NotifyReceiver<DrainRateRequest>,
+    /// The memory quota account to be used for this stream
+    #[deftly(has_memory_cost(indirect_size = "0"))] // estimate (it contains an Arc)
+    pub(crate) memquota: StreamAccount,
+}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -246,14 +337,14 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
     use tor_cell::relaycell::{
-        msg::{Begin, BeginDir, Data, Resolve},
         AnyRelayMsgOuter, RelayCellFormat,
+        msg::{Begin, BeginDir, Data, Resolve},
     };
 
     use super::*;
