@@ -11,8 +11,8 @@ use tor_cell::relaycell::{RelayCmd, UnparsedRelayMsg};
 use tracing::debug;
 
 use crate::congestion::sendme;
+use crate::stream::StreamTarget;
 use crate::stream::queue::StreamQueueReceiver;
-use crate::tunnel::StreamTarget;
 use crate::{Error, Result};
 
 /// The read part of a stream on a particular circuit.
