@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -60,11 +60,9 @@ pub use method::{
 pub use obj::{Object, ObjectArcExt, ObjectId};
 
 #[cfg(feature = "describe-methods")]
-#[cfg_attr(docsrs, doc(cfg(feature = "describe-methods")))]
 pub use dispatch::description::RpcDispatchInformation;
 
 #[cfg(feature = "describe-methods")]
-#[cfg_attr(docsrs, doc(cfg(feature = "describe-methods")))]
 #[doc(hidden)]
 pub use dispatch::description::DelegationNote;
 
@@ -320,7 +318,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
