@@ -7,8 +7,8 @@
 
 #![forbid(unsafe_code)] // if you remove this, enable (or write) miri tests (git grep miri)
 
-use super::io::{stream_pair, LocalStream};
 use super::MockNetRuntime;
+use super::io::{LocalStream, stream_pair};
 use crate::util::mpsc_channel;
 use core::fmt;
 use tor_rtcompat::tls::TlsConnector;
@@ -18,12 +18,12 @@ use tor_rtcompat::{
 use tor_rtcompat::{UdpProvider, UdpSocket};
 
 use async_trait::async_trait;
+use futures::FutureExt;
 use futures::channel::mpsc;
 use futures::io::{AsyncRead, AsyncWrite};
 use futures::lock::Mutex as AsyncMutex;
 use futures::sink::SinkExt;
 use futures::stream::{Stream, StreamExt};
-use futures::FutureExt;
 use std::collections::HashMap;
 use std::fmt::Formatter;
 use std::io::{self, Error as IoError, ErrorKind, Result as IoResult};
@@ -508,10 +508,7 @@ impl TlsConnector<LocalStream> for MockTlsConnector {
         let peer_cert = stream.tls_cert.take();
 
         if peer_cert.is_none() {
-            return Err(std::io::Error::new(
-                std::io::ErrorKind::Other,
-                "attempted to wrap non-TLS stream!",
-            ));
+            return Err(std::io::Error::other("attempted to wrap non-TLS stream!"));
         }
 
         Ok(MockTlsStream { peer_cert, stream })
@@ -595,7 +592,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
